
#include "precomp.h"
#include "chatview.h"
#include "chat.h"
#include "ircconnection.h"

/////////////////////////////////////////////////////////////////////////////
// ChatView construction:

ChatView::ChatView(IRCConnection *connection, const std::string &filter, MainFrame *mainframe) :
	View(mainframe), chat(0), inputbox(0), userlist(0), connection(connection)
{
	chat = new Chat(connection, filter, this);
	inputbox = new CL_InputBox(this);
	userlist = new CL_ListBox(this);
	userlist->show(false);
	inputbox->set_focus();

	slots.connect(sig_resize(), this, &ChatView::on_resize);
	slots.connect(sig_paint(), this, &ChatView::on_paint);
	slots.connect(inputbox->sig_return_pressed(), this, &ChatView::on_inputbox_return_pressed);
	slots.connect(inputbox->sig_changed(), this, &ChatView::on_inputbox_changed);
}

ChatView::~ChatView()
{
}

/////////////////////////////////////////////////////////////////////////////
// ChatView attributes:


/////////////////////////////////////////////////////////////////////////////
// ChatView operations:


/////////////////////////////////////////////////////////////////////////////
// ChatView implementation:

void ChatView::on_resize(int old_width, int old_height)
{
	chat->set_position(CL_Rect(0, 0, get_width(), get_height()-16));
	inputbox->set_position(CL_Rect(0, get_height()-16, get_width(), get_height()));
}

void ChatView::on_paint()
{
}

void ChatView::on_inputbox_return_pressed()
{
	std::string command_line = inputbox->get_text();
	if (command_line.empty()) return;

	if (command_line[0] != '/') // not a command
	{
		connection->send_privmsg("#clanlib", command_line);
		chat->add_line(command_line, chat->color_text);
	}
	else // its a command
	{
		std::vector<std::string> args = CL_String::tokenize(command_line, " ");

		if (args[0] == "/quit")
		{
			connection->send_quit("ClanLib - the biggest overkill library ever written for an IRC client!");
			quit();
		}
		else if (args[0] == "/join")
		{
			if (args.size() == 2)
			{
				connection->send_join(args[1]);
			}
			else if (args.size() == 3)
			{
				connection->send_join(args[1], args[2]);
			}
			else
			{
				chat->add_line("/join syntax: /join <channel> [key]", chat->color_system);
			}
		}
	}

	inputbox->set_text("");
}

void ChatView::on_inputbox_changed(const std::string &text)
{
}
