
#include "precomp.h"
#include "mainframe.h"
#include "document.h"
#include "view.h"
#include "ircconnection.h"
#include "chatview.h"
#include "dlg_connect.h"

/////////////////////////////////////////////////////////////////////////////
// MainFrame construction:

MainFrame::MainFrame(CL_Component *parent, CL_ResourceManager *resources) :
	CL_Frame(CL_Rect(0, 0, CL_Display::get_width(), CL_Display::get_height()), parent),
	resources(resources),
	document(0),
	active_view(-1)
{
	document = new Document;
	slots.connect(sig_paint(), this, &MainFrame::on_paint);
/*
	DlgConnect dlg(this);
	dlg.run();
*/
	IRCConnection *connection = new IRCConnection("irc.freenode.net"/*dlg.get_server()*/);
	document->connections.push_back(connection);
	add_view(new ChatView(connection, "", this));
	connection->send_nick("CTalk");
	connection->send_user("ctalk", "localhost", "localhost", "Anonymous CTalk User");
}

MainFrame::~MainFrame()
{
	delete document;
}

/////////////////////////////////////////////////////////////////////////////
// MainFrame attributes:

Document *MainFrame::get_document()
{
	return document;
}

int MainFrame::get_num_views()
{
	return views.size();
}

View *MainFrame::get_view(int index)
{
	return views[index];
}

View *MainFrame::get_active_view()
{
	if (active_view == -1) return 0;
	return views[active_view];
}

CL_Rect MainFrame::get_view_area()
{
	return CL_Rect(0, 0, get_width(), get_height());
}

CL_ResourceManager *MainFrame::get_resources()
{
	return resources;
}

/////////////////////////////////////////////////////////////////////////////
// MainFrame operations:

void MainFrame::set_active_view(View *view)
{
	set_active_view(find_view_index(view));
}

void MainFrame::set_active_view(int index)
{
	if (active_view != -1) views[active_view]->show(false);
	active_view = index;

	views[active_view]->set_position(get_view_area());
	views[active_view]->show(true);
}

void MainFrame::add_view(View *view)
{
	views.push_back(view);
	set_active_view(views.size()-1);
}

void MainFrame::remove_view(View *view)
{
	views.erase(views.begin() + find_view_index(view));
}

int MainFrame::find_view_index(View *view)
{
	int size = views.size();
	for (int i=0; i<size; i++)
	{
		if (views[i] == view)
		{
			return i;
		}
	}

	return -1;
}

/////////////////////////////////////////////////////////////////////////////
// MainFrame implementation:

void MainFrame::on_paint()
{
	if (active_view == -1) CL_Display::clear(CL_Color(80, 128, 80));
}
