#include <ClanLib/core.h>
#include <ClanLib/gui.h>
#include <ClanLib/guistylesilver.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>
#include <ClanLib/gl.h>

class MyListItem : public CL_ListItem
{
	MyListItem() : CL_ListItem() {};
};

// The main application
class App : public CL_ClanApplication
{
public:
	CL_GUIManager *global_gui;

	int main(int argc, char** argv)
	{
		// Create a console window for text-output if not available
		CL_ConsoleWindow console("Console");
		console.redirect_stdio();

		try
		{
			std::cout << "This example shows how to draw custom listbox items" << std::endl;
			std::cout << std::endl;

			CL_SetupCore::init();
			CL_SetupDisplay::init();
			CL_SetupGL::init();
			CL_SetupGUI::init();

			CL_DisplayWindow window("GUICustomList example", 640, 480);

			// Create the GUI using the default style
			CL_ResourceManager gui_resources("../../Resources/GUIStyleSilver/gui.scr", false);
			CL_StyleManager_Silver style(&gui_resources);
			CL_GUIManager gui(&style);
			global_gui = &gui;

			CL_Button button_quit(CL_Point(10,10), "Quit", &gui);
			
			CL_ListBox listbox(CL_Rect(10, 50, 200, 250), &gui);
			listbox.insert_item("item 1");
			listbox.insert_item("item 2");
			listbox.insert_item("item 3");
			listbox.insert_item("item 4");
			
			// Background draw (needed for modal dialogs)
			CL_Slot slot_paint = gui.sig_paint().connect(this, &App::on_paint);

			// Quit
			CL_Slot slot_quit = button_quit.sig_clicked().connect(this, &App::on_quit);
			CL_Slot slot_window_quit = CL_System::sig_quit().connect(this, &App::on_quit);
			
			// Run the show
			gui.run();

			CL_SetupGL::deinit();
			CL_SetupGUI::deinit();
			CL_SetupDisplay::deinit();
			CL_SetupCore::deinit();
		}
		catch (CL_Error e)
		{
 			std::cout << e.message.c_str() << std::endl;

			// Display console close message and wait for a key
			console.display_close_message();
		}

		return 0;
	}

	void on_paint()
	{
		CL_Display::clear(CL_Color(240, 242, 244));
	}

	void on_quit()
	{
		global_gui->quit();
	}
} app;
