/*
	Demonstration of the CL_Mouse interface
*/

#include <ClanLib/core.h>
#include <ClanLib/display.h>
#include <ClanLib/application.h>
#include <ClanLib/gl.h>
#include <strstream>

class MouseSignalApp : public CL_ClanApplication
{
private:
	bool quit;
public:
	void on_mouse_press(const CL_InputEvent &key)
	{
		// Use the same code as on_button_press, but only MOUSE events.
	}

	void window_close()
	{
		quit = true;
	}

	void on_button_press(const CL_InputEvent &key)
	{
		std::strstream button;

		switch(key.id) {
		case CL_MOUSE_LEFT:
			button << "Mouse left button";
			break;
		case CL_MOUSE_MIDDLE:
			button << "Mouse middle button";
			break;
		case CL_MOUSE_RIGHT:
			button << "Mouse right button";
			break;
		case CL_MOUSE_XBUTTON1:
			button << "Mouse thumb button";
			break;
		case CL_MOUSE_XBUTTON2:
			button << "Mouse wheel button";
			break;
		case CL_MOUSE_WHEEL_UP:
			button << "Mouse wheel up";
			break;
		case CL_MOUSE_WHEEL_DOWN:
			button << "Mouse wheel down";
			break;
		case CL_KEY_LCONTROL:
			button << "Left control";
			break;
		case CL_KEY_ESCAPE:
			quit = true;
			break;
		default:
			button << key.id << " (" << key.str << ")";
		}

		button << " pressed at " << key.mouse_pos.x << "," << key.mouse_pos.y << '\0';
		std::cout << button.str() << std::endl;
	}

	virtual int main(int, char **)
	{
		// Create a console window for text-output if not available
		CL_ConsoleWindow console("Console");
		console.redirect_stdio();
		quit = false;


		try
		{
			CL_SetupCore::init();
			CL_SetupDisplay::init();
			CL_SetupGL::init();

			// Set mode: 320x240 16 bpp
			CL_DisplayWindow window("ClanLib MouseSignal Test", 800, 600);

			// Make sure the display is black at startup:
			CL_Display::clear();

			// Open the resource file containing our custom mouse cursor resource
			CL_ResourceManager *manager = new CL_ResourceManager("mouse.scr", false);
			
			// Set the mouse cursor to use a custom mouse provided in our resource file
			CL_Sprite cursor("sur_cursor", manager);
			//window.set_cursor(cursor);
			//CL_Mouse::set_cursor(CL_MouseCursorProvider::load("sur_cursor", manager));

			// This connects ANY input (joystick, mouse, keyboard)
			CL_Slot slot_input_up = window.get_ic()->get_mouse().sig_key_up().connect( this, &MouseSignalApp::on_button_press);

			// Connect the Window close event
			CL_Slot slot_quit = window.sig_window_close().connect(this, &MouseSignalApp::window_close);

			// Loop until the user presses Escape
			while (!quit)
			{
				// Clear display
				CL_Display::clear(CL_Color(0,0,127));

				// Flip front and backbuffer - this makes the changes visible
				CL_Display::flip();

				CL_System::sleep(10);

				// Update keyboard input and handle system events
				CL_System::keep_alive();
			}

			// Delete resource manager
			delete manager;
		}
		catch (CL_Error err)
		{
			std::cout << "Exception caught: " << err.message.c_str() << std::endl;
		}

		CL_SetupDisplay::deinit();
		CL_SetupCore::deinit();

		// Display console close message and wait for a key
		console.display_close_message();

		return 0;
	}
} app;
