
#include "precomp.h"
#include "application.h"
#include "world.h"

Application app;

int Application::main(int argc, char** argv)
{
	CL_ConsoleWindow console("Debug console window", 80, 25);
	console.redirect_stdio();

	try
	{
		std::cout << "ClanLib Pacman game, written by Magnus Norddahl" << std::endl;
		
		CL_SetupCore::init();
		CL_SetupDisplay::init();
		CL_SetupGL::init();
		CL_SetupSound::init();

		if(argc > 1 && strcmp(argv[1],"-datafile") == 0)
		{
			CL_DatafileCompiler::write("pacman.scr", "pacman.dat");
			return 0;
		}
		
		CL_ResourceManager resources;
/*		try
		{
			resources = CL_ResourceManager("pacman.dat", true);
		}
		catch (CL_Error err)
*/		{
			resources = CL_ResourceManager("pacman.scr", false);
		}

		CL_DisplayWindow window("The ClanLib Pacman game!", 640, 480);

		World world(&resources);
		world.run(window);
	}
	catch (CL_Error err)
	{
		std::cout << "Exeception caught: " << err.message.c_str() << std::endl;
	}

	CL_SetupSound::deinit();
	CL_SetupGL::deinit();
	CL_SetupDisplay::deinit();
	CL_SetupCore::deinit();
	
	console.wait_for_key();

	return 0;
}
