#include <stdio.h>
#include <ClanLib/gl.h>

#include "app.h"

Application app;

int Application::main(int argc, char** argv)
{
	// Create a console window for text-output if not available
	CL_ConsoleWindow console("Console");
	console.redirect_stdio();

	try
	{
		CL_SetupCore::init();
		CL_SetupGL::init();
		CL_SetupDisplay::init();
		
		CL_DisplayWindow window("Line Demo", 640, 480);
		
		logo = new CL_Surface("Logo.png");

		// Antialias lines
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
		glEnable(GL_LINE_SMOOTH);

		run();

		CL_SetupDisplay::deinit();
		CL_SetupGL::deinit();
		CL_SetupCore::deinit();
	}
	catch (CL_Error e)
	{
		std::cout << e.message.c_str() << std::endl;

		// Display console close message and wait for a key
		console.display_close_message();
	}

	return 0;
}

void Application::run()
{
	do {
		update();
		draw();

		CL_Display::flip();
		CL_System::keep_alive(10);
	} while(CL_Keyboard::get_keycode(CL_KEY_ESCAPE) == false);
}

void Application::draw()
{
	static int x = 0;
	static int y = 0;
	static int count = 0;

	// Fade background
	CL_Display::clear(CL_Color(0,0,0,5));

	// Draw background "smoke"
	for(int p = 0; p < 10; p++)
	{
		int x = rand() % 640;
		int y = rand() % 480;
		CL_Display::fill_rect(CL_Rect(x - 150, y - 150, x + 150, y + 150), CL_Color(255, 255, 255, 10));
	}

	// Draw lines
	for(int l = 0; l < LINE_COUNT; l++)
		lines[l].draw();

	// Move logo around ?
	if(rand() % 100 == 0 || count)
	{
		if(count == 0)
			count = 30;
		else
			count--;
		x = rand() % 400 - 200;
		y = rand() % 400 - 200;
	}

	// Draw logo
	logo->draw((320 - 105) + x, (240 - 22) + y);
}

void Application::update()
{
	for(int i=0; i<LINE_COUNT; i++)
		lines[i].update();
}
