#include <ClanLib/core.h>
#include <ClanLib/application.h>

class XMLTest : public CL_ClanApplication
{
public:
	virtual int main(int argc, char **argv)
	{
		// Create a console window for text-output if not available
		CL_ConsoleWindow console("Console");
		console.redirect_stdio();
	
		try
		{
			// Initialize ClanLib base components
			CL_SetupCore::init();

			// Create xml document
			CL_XmlDocument xml_file;

			// Create xml element (can be container)
			CL_XmlElement elem("sample");

			// Add some atributes to it
			elem.SetAttribute("test_value", 14);
			elem.SetAttribute("another_value", "string value");

			// Create another xml element (it will be child of first element)
			CL_XmlElement elem2("item");

			// Add some atributes to it
			elem2.SetAttribute("int val", 10);
			elem2.SetAttribute("str val", "str data");

			// Connect it with parent
			elem.InsertEndChild(elem2);

			// Connect first element with xml document
			xml_file.InsertEndChild(elem);

			// Finaly, save xml document to file
			xml_file.SaveFile("test.xml");

			std::cout << "look at test.xml in this folder" << std::endl;
			console.wait_for_key();

			CL_SetupCore::deinit();
		}
		catch(CL_Error error)
		{
			std::cout << "Exception caught : " << error.message.c_str() << std::endl;

			// Display console close message and wait for a key
			console.display_close_message();

			return -1;
		}

		return 0;
	}
} my_app;
