/*
	$Id: cl_endian.h,v 1.11 2002/07/16 12:39:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="I/O Data"
//! header=core.h

#ifndef header_endian
#define header_endian

//: Endianess management class.
//- <p>The CL_Endian class is used to convert between different endianess. There
//- are also a set of defines to the lazy developer:</p>
//- <p>#define SWAP_IF_BIG(i)<br>
//- #define SWAP_IF_LITTLE(i)<br>
//- #define SWAP_IF_BIG_ALOT(i, times)<br>
//- #define SWAP_IF_LITTLE_ALOT(i, times)<br>
//- #define IS_SYSTEM_64BIT()</p>
//- <p>Each of these defines call CL_Endian::is_system_big() and CL_Endian::swap.</p>
class CL_Endian
{
public:
//! Operations:
	//: Swaps larger amounts of data between little and big endian.
	//param data: Data to be swapped.
	//param type_size: Size of datatype to be swapped.
	//param total_times: Number of 'type_size' size data chunks to be swapped.
	static void swap(void *data, int type_size, int total_times=1);

//! Attributes:
	//: Returns true if big endian system.
	//return: True if big endian system, false otherwise.
	static bool is_system_big();

	//: Returns true if 64 bit system.
	//return: True if 64 bit, false otherwise.
	static bool is_system_64bit();
};

// Defines to the lazy developer:
#define SWAP_IF_BIG(i) if (CL_Endian::is_system_big()) CL_Endian::swap(&i, sizeof(i));
#define SWAP_IF_LITTLE(i) if (!CL_Endian::is_system_big()) CL_Endian::swap(&i, sizeof(i));

#define SWAP_IF_BIG_ALOT(i, times) if (CL_Endian::is_system_big()) CL_Endian::swap(&i, sizeof(i), times);
#define SWAP_IF_LITTLE_ALOT(i, times) if (!CL_Endian::is_system_big()) CL_Endian::swap(&i, sizeof(i), times);

#define IS_SYSTEM_64BIT() CL_Endian::is_system_64bit();

#endif
