/*
	$Id: outputsource_memory.h,v 1.15 2002/07/16 12:39:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="I/O Data"
//! header=core.h

#ifndef header_outputsource_memory
#define header_outputsource_memory

#include "outputsource.h"

//: Outputsource writing to memory.
//- <p>CL_OutputSource_Memory provides a way to write to memory in a way that is abstracted from 
//- the user. CL_OutputSource_Memory should not be instantiated directly, but 
//- rather CL_OutputSourceProvider should create the appropriate output source.</p>

class CL_OutputSource_Memory : public CL_OutputSource
{
//! Construction:
public:
	//: Constructs a memory output source.
	CL_OutputSource_Memory();

	//: Output Source Memory destructor.
	virtual ~CL_OutputSource_Memory();

//! Attributes:
public:
	//: Returns current position in output source.
	//return: Current position in source.
	virtual int tell() const;

	//: Returns the size of the output source.
	//return: Size of the output source.
	virtual int size() const;

	//: Get the pointer to the internal data array in the memory source.
	//- <p>Used to extract the data after usage.</p>
	//return: String containing the data written to the output source.
	std::string get_data() const;

//! Operations:
public:
	//: Writes larger amounts of data (no endian and 64 bit conversion).
	//param data: Points to the array from which to write.
	//param size: Number of bytes to write.
	//return: Num bytes actually written.
	virtual int write(const void *data, int size);
	
	//: Opens the output source. By default, it is open.
	virtual void open();
	
	//: Closes the output source.
	virtual void close();

	//: Make a copy of the current outputsource, standing at the same position.
	//return: The copy of the output source.
	virtual CL_OutputSource *clone();

//! Implementation:
private:
	std::string data;
	unsigned int pos;
};

#endif
