/*
	$Id: outputsource_zipped.h,v 1.12 2002/07/16 12:39:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="I/O Data"
//! header=core.h

#ifndef header_outputsource_zipped
#define header_outputsource_zipped

#include "outputsource.h"

class CL_OutputSource_Zipped_Generic;

//: Zlib compressed output source.
//- <p>CL_OutputSource_Zipped_Generic provides write access to compress files while internally
//-  dealing with endianess.</p>
//- <p>CL_OutputSource_Zipped_Generic should not be instantiated directly, but
//- rather CL_OutputSourceProvider should create the appropriate output source.</p>

class CL_OutputSource_Zipped : public CL_OutputSource
{
//! Construction:
public:
	//: Constructs an output source that zips the data and passes it on to
	//: the passed output source.
	//- <p>If delete_output is true, the zipped output source will delete the
	//- output source at destruction.</p>
	CL_OutputSource_Zipped(CL_OutputSource *output, bool delete_output = false);

	CL_OutputSource_Zipped(const CL_OutputSource_Zipped &copy);

	//: Destructor.
	virtual ~CL_OutputSource_Zipped();

//! Attributes:
public:
	//: Returns current position in output source.
	//return: Current position in source.
	virtual int tell() const;

	//: Returns the size of the output source
	//return: Size of the output source.
	virtual int size() const;

//! Operations:
public:
	//: Writes larger amounts of data (no endian and 64 bit conversion):
	//param data: Points to the array from which to write.
	//param size: Number of bytes to write.
	//return: Num bytes actually written.
	virtual int write(const void *data, int size);
	
	//: Opens the output source. By default, it is open.
	virtual void open();
	
	//: Closes the output source.
	virtual void close();

	//: Make a copy of the current outputsource, standing at the same position.
	//return: The copy of the output source.
	virtual CL_OutputSource *clone();

//! Implementation:
private:
	CL_OutputSource_Zipped_Generic *impl;
};

#endif
