/*
	$Id: resourcedata.h,v 1.9 2002/07/16 12:39:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="Resources"
//! header=core.h

#ifndef header_resourcedata
#define header_resourcedata

#include "resource.h"

class CL_ResourceData_Generic;

//: Data object attached to a resource.
//- <p>The CL_ResourceData class represents a data object connected to a
//- resource. A resource can have one or more of these data objects
//- attached, and they can be retrieved by making a call to
//- CL_Resource::get_data.</p>
//- <p>Resource data objects are used to contain extra data associated with
//- a certain resource. For instance, the surface resource has an object
//- attached by the name "surface" that contain the loaded surface's
//- data.</p>
class CL_ResourceData
{
//! Construction:
public:
	//: Constructs a resource data object.
	//- <p>A resource data object will not by itself attach to a
	//- CL_Resource. This must be done manually by making a call to
	//- CL_Resource::attach_data.</p>
	//param resource: Resource object associated with this resource data
	//param resource: object.
	CL_ResourceData(CL_Resource &resource);

	//: Resource Data destructor.
	virtual ~CL_ResourceData();

//! Attributes:
public:
	//: Get Resource object.
	CL_Resource get_resource();

//! Implementation:
private:
	//: Pointer to the implementation.
	CL_ResourceData_Generic *impl;
};

#endif
