/*
	$Id: resourceoption.h,v 1.19 2002/08/25 18:45:43 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="Resources"
//! header=core.h

#ifndef header_resourceoption
#define header_resourceoption

#include <vector>
#include <string>

//: Resource option class.
//- <p>Holds the particular option for a resource.</p>
class CL_ResourceOption
{
//! Construction:
public:
	//: Constructs a resource option.
	//param name: The name of the option
	//param value: Value of the option as a string
	CL_ResourceOption(const std::string &name, const std::string &value) : name(name)
	{
		num_vals = 1;
		values.push_back(value);
	}

	CL_ResourceOption(const std::string &name) : name(name)
	{
		num_vals = 0;
	}

	CL_ResourceOption()
	{
		num_vals = 0;
	}

	CL_ResourceOption(const CL_ResourceOption &other)
	{
		num_vals = other.num_vals;
		name = other.name;
		if(num_vals)
		{
			std::vector<std::string>::const_iterator it;
			for(it = other.values.begin(); it != other.values.end(); ++it)
			{
				values.push_back(*it);
			}
		}
	}

//! Attributes:
public:
	//: Numbers of values available for this option.
	//return: The number of options available.
	int get_value_count() const { return (int)values.size(); }

	//: Gets the name of the resource option.
	//return: The name of the option.
	const std::string &get_name() const { return name; }

	//: Gets a value of from resource option.
	//param index: The index of which value to return, default of 0.
	//return: The option value.
	const std::string &get_value(int index = 0) const { return values[index]; }

	//: Gets all values as a vector.
	//return: An std::vector<std::string> with all the values.
	std::vector<std::string> &get_values() { return values; }

//! Operations:
public:
	void operator =( const CL_ResourceOption &other)
	{
		num_vals = other.num_vals;
		name = other.name;
		if(num_vals)
		{
			std::vector<std::string>::const_iterator it;
			for(it = other.values.begin(); it != other.values.end(); ++it)
			{
				values.push_back(*it);
			}
		}
	}

	//: Add a value.
	//param value: The string value to add.
	//return: The index of the added value.
	int add_value(const std::string &value) { values.push_back(value); return num_vals++; }

//! Implementation:
private:
	int num_vals;
	std::string name;
	std::vector<std::string> values;
};

#endif
