/*
	$Id: resourcetype_raw.h,v 1.14 2002/07/16 12:39:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="Resources"
//! header=core.h

#ifndef header_resourcetype_raw
#define header_resourcetype_raw

#include "resourcedata.h"
#include "resource_manager.h"
#include "resource.h"
#include "../IOData/inputsource_memory.h"
#include <string>
#include <vector>

//: Resource Data object for a 'raw' resource type.
class CL_ResourceData_Raw : public CL_ResourceData
{
//! Construction:
public:
	//: Resource Data Raw Constructor.
	CL_ResourceData_Raw(class CL_Resource &resource);

//! Attributes:
public:
	//: Vector array containing the raw data.
	std::vector<unsigned char> value;

//! Implementation:
private:
	void on_load_file();
	void on_load_datafile(class CL_InputSourceProvider *provider);
	void on_save_datafile(class CL_OutputSourceProvider *provider);
	void on_unload();

	CL_Slot slot_load_file;
	CL_Slot slot_load_datafile;
	CL_Slot slot_save_datafile;
	CL_Slot slot_unload;
};

//: Raw resource accessor class.
class CL_Raw
{
//! Construction:
public:
	//: Constructs an raw object.
	//param res_id: Resource id of an raw resource to load from.
	//param resources: Resource manager to load resource from.
	CL_Raw(const std::string &res_id, CL_ResourceManager *resources)
	{
		res = resources->get_resource(res_id);
		res.load();
		data = (CL_ResourceData_Raw *) res.get_data("raw");
	}

	//: Raw Class destructor.
	~CL_Raw()
	{
		res.unload();
	}

//! Attributes:
public:
	//: Returns the raw data as a vector.
	std::vector<unsigned char> &get_data()
	{
		return data->value;
	}

	const std::vector<unsigned char> &get_data() const
	{
		return data->value;
	}

private:
	CL_Resource res;
	CL_ResourceData_Raw *data;
};

//: Input source interface for 'raw' resource types.
class CL_InputSource_Raw : public CL_Raw, public CL_InputSource_Memory
{
public:
	//: Construct input source reading from a 'raw' resource type.
	//param res_id: Resource id of an raw resource to load from.
	//param resources: Resource manager to load resource from.
	CL_InputSource_Raw(const std::string &res_id, CL_ResourceManager *resources)
		: CL_Raw(res_id, resources), CL_InputSource_Memory(std::string((char *) &CL_Raw::get_data()[0], CL_Raw::get_data().size()))
	{
	}
};

#endif
