/*
	$Id: event_trigger.h,v 1.16 2002/07/16 12:39:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="System"
//! header=core.h

#ifndef header_event_trigger
#define header_event_trigger

class CL_EventListener;
class CL_EventTrigger_Generic;

//: ClanLib Event trigger.
//- <p>This class represents an event that can be triggered.</p>
//- <p>Collect several triggers together using the CL_EventListener class, and then
//- call the CL_EventListener::wait function to sleep until one of the events
//- are triggered.</p>
class CL_EventTrigger
{
//! Construction:
public:
	//: Constructs an event trigger object.
	CL_EventTrigger();

	CL_EventTrigger(const CL_EventTrigger &copy);

	//: Event Trigger destructor.
	virtual ~CL_EventTrigger();

//! Attributes:
public:
	//: Returns true if the trigger flag has been raised.
	bool get_flag() const;

//! Operations:
public:
	//: Assignment operator.
	CL_EventTrigger &operator =(const CL_EventTrigger &copy);

	//: Reset trigger flag.
	void reset() const;

	//: Raise trigger flag.
	void set_flag();

	//: Wait until the event triggers.
	//param timeout - The timeout is in milliseconds. If -1, waits infinitely.
	//- <p>If timeout = -1, this function will not timeout.</p>
	//returns: True if event triggered, false if it timed out.
	bool wait(int timeout = -1);

//! Implementation:
public:
	CL_EventTrigger_Generic *impl;
};

#endif
