/*
	$Id: system.h,v 1.12 2002/08/09 13:06:42 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanCore="System"
//! header=core.h

#ifndef header_system
#define header_system

class CL_EventListener;
class CL_Signal_v0;

//: Generic functions that didn't fit in other places.
class CL_System
{
//! Attributes:
public:
	//: Get the current time (since system boot), in milliseconds.
	static unsigned int get_time();

	//: Returns true if MMX technology is available on this CPU.
	static bool detect_mmx();
	
	//: Returns true if 3DNow! technology is available on this CPU.
	static bool detect_3dnow();

	//: Returns true if extended 3DNow! is present on this CPU.
	static bool detect_ext_3dnow();

//! Operations:
public:
	//: Checks for system events, updates input and other stuff.
	static void keep_alive();

	//: Checks for system events (pump the CL_KeepAlive objects).
	//- <p>Return when 'millis' has elapsed.</p>
	static void keep_alive(int millis);

	//: Checks for system events (pump the CL_KeepAlive objects).
	//- <p>Return when one of the events trigger, or when timeout has elapsed.</p>
	static bool keep_alive(CL_EventListener &events, int timeout = -1);

	//: Sleep for 'millis' milliseconds.
	static void sleep(int millis);

	//: Suspend the time.
	//- <p>get_time() will continue to return the same value until resume_time() is called.</p>
	static void suspend_time();

	//: Resumes the time.
	//- <p>get_time() will return the current time again.</p>
	static void resume_time();

//! Signals:
public:
	//: Returns the signal emitted when WM_QUIT or similar window system quit
	//: message is received.
	static CL_Signal_v0 &sig_quit();
};

#endif
