/*
	$Id: direct3d_window.h,v 1.8 2002/07/17 13:48:49 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanDX8="Display 2D"
//! header=dx8.h

#ifndef header_direct3d_window
#define header_direct3d_window
#ifdef WIN32

#include "../Display/display_window.h"
#include <d3d8.h>

//: Direct3D top level window.
class CL_Direct3DWindow : public CL_DisplayWindow
{
//! Construction:
public:
	//: Constructs a direct3d window.
	//param title: Titlebar text.
	//param width, height: Size of window, in pixels.
	//param start_fullscreen: If true, will show window full screen.
	//param allow_resize: If true, window will have resize grabbers.
	//param flipping_buffers: Number of buffers in flipping system.
	//param device_type, behavior_flags, present_parameters: See the IDirect3D8::CreateDevice function for more details.
	CL_Direct3DWindow(
		const std::string &title,
		int width,
		int height,
		bool start_fullscreen = false,
		bool allow_resize = false,
		int flipping_buffers = 2);

	CL_Direct3DWindow(
		const std::string &title,
		int width,
		int height,
		bool allow_resize,
		D3DDEVTYPE device_type,
		DWORD behavior_flags,
		D3DPRESENT_PARAMETERS *present_parameters);

	~CL_Direct3DWindow();

//! Attributes:
public:
	//: Returns the direct3d object associated with this window.
	LPDIRECT3D8 get_direct3d();

	//: Returns the direct3d device associated with this window.
	IDirect3DDevice8 *get_device();

	//: Returns the Win32 window handle.
	HWND get_wnd();

//! Operations:
public:

//! Implementation:
private:
};

#endif
#endif
