/*
	$Id: provider_factory.h,v 1.7 2002/08/20 10:41:24 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanDisplay="Surface Providers"
//! header=display.h

#ifndef header_provider_factory
#define header_provider_factory

#include <map>
#include <string>

class CL_PixelBuffer;
class CL_InputSourceProvider;
class CL_ProviderType;

//: Provider factory
class CL_ProviderFactory
{
//! Static Attributes:
public:
	//: Map of the class factories for each provider type.
	static std::map<std::string, CL_ProviderType *> types;

//! Operations:
public:
	//: Loads an image file from 'filename', using the provider
	//: identified by 'type'.
	//- <p>If the type is an empty string, it uses the extension of
	//- the filename to determine what type it is.</p>
	//- <p>The input provider will be used as a the source for filename,
	//- if specified.</p>
	static CL_PixelBuffer *load(
		const std::string &filename,
		const std::string &type = "",
		CL_InputSourceProvider *input_provider = 0);
};

#endif
