/*
	$Id: joystick.h,v 1.1 2002/08/02 22:43:39 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanDisplay="Input"
//! header=display.h

#ifndef header_joystick
#define header_joystick

#ifdef WIN32
#pragma warning( disable : 4786)
#endif

#include <string>
#include "../signals.h"

class CL_InputEvent;
class CL_InputDevice;

//: 
class CL_Joystick
{
//! Construction:
public:

//! Attributes:
public:
	//: Returns the number of joysticks available.
	static int get_device_count();

	//: Returns a joystick device.
	static CL_InputDevice &get_device(int joystick = 0);
	
	//: Returns true if the key with the specified key code is pressed.
	static bool get_keycode(int keycode, int joystick = 0);

	//: Key name for specified identifier (Joystick Button 1).
	static std::string get_key_name(int id);

//! Operations:
public:

//! Signals:
public:
	//: Signal emitted when a joystick button is pressed on the specified joystick.
	static CL_Signal_v1<const CL_InputEvent &> &sig_key_down(int joystick = 0);

	//: Signal emitted when a joystick button is released on the specified joystick.
	static CL_Signal_v1<const CL_InputEvent &> &sig_key_up(int joystick = 0);

	//: Signal emitted when a joystick button is double clicked.
	static CL_Signal_v1<const CL_InputEvent &> &sig_key_dblclk(int joystick = 0);

	//: Signal emitted when specified joystick is moved.
	static CL_Signal_v1<const CL_InputEvent &> &sig_move(int joystick = 0);

//! Implementation:
private:
};

#endif
