/*
	$Id: keyboard.h,v 1.10 2002/08/02 22:43:39 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanDisplay="Input"
//! header=display.h

#ifndef header_keyboard
#define header_keyboard

#ifdef WIN32
#pragma warning( disable : 4786)
#endif

#include <string>
#include "../signals.h"

class CL_InputEvent;
class CL_InputDevice;

//: 
class CL_Keyboard
{
//! Construction:
public:

//! Attributes:
public:
	//: Returns the number of keyboards available.
	static int get_device_count();

	//: Returns a keyboard device.
	static CL_InputDevice &get_device(int keyboard = 0);

	//: Returns true if the key with the specified key code is pressed.
	static bool get_keycode(int keycode, int keyboard = 0);

	//: Key name for specified identifier (A, B, Space, Backspace).
	static std::string get_key_name(int id);

//! Operations:
public:

//! Signals:
public:
	//: Signal emitted when key is pressed on the specified keyboard.
	static CL_Signal_v1<const CL_InputEvent &> &sig_key_down(int keyboard = 0);

	//: Signal emitted when key is released on the specified keyboard.
	static CL_Signal_v1<const CL_InputEvent &> &sig_key_up(int keyboard = 0);

	//: Signal emitted when key is double clicked on the specified keyboard.
	static CL_Signal_v1<const CL_InputEvent &> &sig_key_dblclk(int keyboard = 0);

//! Implementation:
private:
};

#endif
