/*
	$Id: soundprovider_factory.h,v 1.2 2002/07/15 16:49:50 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanSound="Sound Providers"
//! header=sound.h

#ifndef header_soundprovider_factory
#define header_soundprovider_factory

#include <map>
#include <string>

class CL_StaticSoundProvider;
class CL_StreamSoundProvider;
class CL_InputSourceProvider;
class CL_SoundProviderType;

class CL_SoundProviderFactory
{
//! Attributes:
public:
	//: Map of the class factories for each sound provider type.
	static std::map<std::string, CL_SoundProviderType *> types;

//! Operations:
public:
	//: Loads an sample file from 'filename', using the provider
	//: identified by 'type'.
	//- <p>If the type is an empty string, it
	//- uses the extension of the filename to determine what type
	//- it is.</p>
	//- <p>The input provider will be used as a the source for filename,
	//- if specified.</p>
	static CL_StaticSoundProvider *load_static(
		const std::string &filename,
		const std::string &type = "",
		CL_InputSourceProvider *input_provider = 0);

	//: Loads an sample file from 'filename', using the provider
	//: identified by 'type'.
	//- If the type is an empty string, it
	//- uses the extension of the filename to determine what type
	//- it is.</p>
	//- <p>The input provider will be used as a the source for filename,
	//- if specified.</p>
	static CL_StreamSoundProvider *load_stream(
		const std::string &filename,
		const std::string &type = "",
		CL_InputSourceProvider *input_provider = 0);
};

#endif
