/*
	$Id: soundprovider_type.h,v 1.2 2002/07/15 16:49:50 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanSound="Sound Providers"
//! header=sound.h

#ifndef header_soundprovider_type
#define header_soundprovider_type

#include <string>

class CL_StaticSoundProvider;
class CL_StreamSoundProvider;
class CL_InputSourceProvider;

//: Sound provider type
class CL_SoundProviderType
{
//! Construction:
public:
	//: Registers a sound provider type in the CL_SoundProviderFactory.
	CL_SoundProviderType(const std::string &type);

	//: Unregisters a sound provider type in the CL_SoundProviderFactory.
	virtual ~CL_SoundProviderType();

//! Operations:
public:
	//: Called to load static with this sound provider type.
	virtual CL_StaticSoundProvider *load_static(
		const std::string &filename,
		CL_InputSourceProvider *input_provider) { return 0; }

	//: Called to load stream with this sound provider type.
	virtual CL_StreamSoundProvider *load_stream(
		const std::string &filename,
		CL_InputSourceProvider *input_provider) { return 0; }

//! Implementation:
private:
};

#endif
