/*
	$Id: soundprovider_type_register.h,v 1.1 2002/07/13 00:18:18 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanSound="Sound Providers"
//! header=sound.h

#ifndef header_soundprovider_type_register
#define header_soundprovider_type_register

#include "soundprovider_type.h"

//: Class template to register a sound provider type.
template<class StaticSoundProviderClass, class StreamSoundProviderClass>
class CL_SoundProviderType_Register : public CL_SoundProviderType
{
//! Construction:
public:
	//: Registers sound provider type in the CL_SoundProviderFactory.
	CL_SoundProviderType_Register(const std::string &type) : CL_SoundProviderType(type)
	{
	}

//! Operations:
public:
	//: Called to load static with this sound provider type.
	virtual CL_StaticSoundProvider *load_static(
		const std::string &filename,
		CL_InputSourceProvider *input_provider)
	{
		return new StaticSoundProviderClass(filename, input_provider);
	}

	//: Called to load stream with this sound provider type.
	virtual CL_StreamSoundProvider *load_stream(
		const std::string &filename,
		CL_InputSourceProvider *input_provider)
	{
		return new StreamSoundProviderClass(filename, input_provider);
	}
};

//: Class template to register a static sound provider type.
template<class SoundProviderClass>
class CL_SoundProviderType_RegisterStatic : public CL_SoundProviderType
{
//! Construction:
public:
	//: Registers sound provider type in the CL_SoundProviderFactory.
	CL_SoundProviderType_RegisterStatic(const std::string &type) : CL_SoundProviderType(type)
	{
	}

//! Operations:
public:
	//: Called to load static with this sound provider type.
	virtual CL_StaticSoundProvider *load_static(
		const std::string &filename,
		CL_InputSourceProvider *input_provider)
	{
		return new SoundProviderClass(filename, input_provider);
	}

	//: Called to load stream with this sound provider type.
	virtual CL_StreamSoundProvider *load_stream(
		const std::string &filename,
		CL_InputSourceProvider *input_provider)
	{
		return 0;
	}
};

//: Class template to register a stream sound provider type.
template<class SoundProviderClass>
class CL_SoundProviderType_RegisterStream : public CL_SoundProviderType
{
//! Construction:
public:
	//: Registers sound provider type in the CL_SoundProviderFactory.
	CL_SoundProviderType_RegisterStream(const std::string &type) : CL_SoundProviderType(type)
	{
	}

//! Operations:
public:
	//: Called to load static with this sound provider type.
	virtual CL_StaticSoundProvider *load_static(
		const std::string &filename,
		CL_InputSourceProvider *input_provider)
	{
		return 0;
	}

	//: Called to load stream with this sound provider type.
	virtual CL_StreamSoundProvider *load_stream(
		const std::string &filename,
		CL_InputSourceProvider *input_provider)
	{
		return new SoundProviderClass(filename, input_provider);
	}
};

#endif
