/*
	$Id: voice_decoder.h,v 1.6 2002/07/15 16:49:51 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

//! clanVoice="Encoding"
//! header=ttf.h

#ifndef header_voice_decoder
#define header_voice_decoder

#include "../Sound/stream_soundprovider.h"
#include <string>

class CL_VoiceDecoder_Generic;

//: Voice decoder
class CL_VoiceDecoder : public CL_StreamSoundProvider_Session
{
//! Construction:
public:
	//: Construct voice decoder.
	CL_VoiceDecoder();

	~CL_VoiceDecoder();

//! Attributes:
public:
	//: Returns the playback frequency of the input data.
	//return: Playback frequency.
	virtual int get_frequency() const;

	//: Returns the playback sample format.
	//return: The playback Sample format.
	virtual SoundFormat get_format() const;

//! Operations:
public:
	// Decodes packet and makes it available through the stream provider session.
	void decode_packet(const std::string &packet);

	//: Called when a playback session needs more sample data.
	//param data_ptr: Points to a buffer that should be filled with sample data.
	//param data_requested: Bytes of data requested.
	//return: Number of bytes actually filled with sample data.
	virtual int get_data(void *data_ptr, int data_requested);

	//: Returns true if no more input data is available.
	//return: True if end of input data. False otherwise.
	virtual bool eof() const;
	
	//: Stops the current stream.
	virtual void stop();

	//: Start/continue playing of the stream.
	//return: True for success. False otherwise.
	virtual bool play();
		
	//: Sets the position within the current stream.
	//param pos: Position to seek to.
	//return: True for success. False otherwise.
	virtual bool set_position(int pos);

//! Implementation:
private:
	CL_VoiceDecoder_Generic *impl;
};

#endif
