/*
	$Id: outputsource_memory_generic.h,v 1.5 2002/07/17 14:55:29 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_outputsource_memory_generic
#define header_outputsource_memory_generic

#include "API/Core/IOData/outputsource.h"

//: Outputsource writing to memory_generic.
class CL_OutputSource_MemoryGeneric : public CL_OutputSource
{
public:
	CL_OutputSource_MemoryGeneric(unsigned int blocksize = 4*1024);
	virtual ~CL_OutputSource_MemoryGeneric();
	
	//: Get the pointer to the internal data array in the memory_generic source.
	//: Used to extract the data after usage.
	//return: Pointer to the data written.
	void *get_data() const;

	//: Writes larger amounts of data (no endian and 64 bit conversion):
	//param data: Points to the array written.
	//param size: Number of bytes to write.
	//return: Num bytes actually written.
	virtual int write(const void *data, int size);
	
	//: Opens the output source. By default, it is open.
	virtual void open();
	
	//: Closes the output source.
	virtual void close();

	//: Make a copy of the current OutputSource, standing at the same position.
	//return: The copy of the output source.
	virtual CL_OutputSource *clone();

	//: Returns current position in output source.
	//return: Current position in source.
	virtual int tell() const;

	//: Returns the size of the output source
	//return: Current size of output source.
	virtual int size() const;

	//: Cleans out the databuffer without deleting it.
	virtual void purge();

private:
	unsigned int m_blocksize;
	unsigned char *m_data;
	unsigned int m_size, m_pos;
};

#endif
