/*
	$Id: directory_scanner_win32.h,v 1.9 2002/11/29 02:58:05 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

#ifndef header_directory_scanner_win32
#define header_directory_scanner_win32

#include "../Generic/directory_scanner_generic.h"

#include <io.h>
#include <direct.h>

class CL_DirectoryScanner_Win32: public CL_DirectoryScanner_Generic
{
//! Construction:
public:
	//: Construct initial directory scanner.
	CL_DirectoryScanner_Win32();

	//: Copy constructor.
	CL_DirectoryScanner_Win32(const CL_DirectoryScanner_Win32 &copy);

	virtual ~CL_DirectoryScanner_Win32();

//! Attributes:
public:
	//: Scan for files in a directory.
	virtual bool scan (const std::string &pathname);

	//: Scan for files matching a pattern.
	virtual bool scan (const std::string &pathname, const std::string &pattern);

	//: Returns the path of the directory being scanned.
	virtual std::string get_directory_path();

	//: Returns the name of the current found file.
	virtual std::string get_name();
	
	//: Returns the name of the current found file, including the directory path.
	virtual std::string get_pathname();
	
	//: Returns true if filename is a directory.
	virtual bool is_directory();

	//: Returns true if filename is hidden.
	virtual bool is_hidden();

	//: Returns true if filename is readable.
	virtual bool is_readable();

	//: Returns true if filename is writable.
	virtual bool is_writable();

//! Operations:
public:
	//: Find next file in directory scan. Returns false if no more files was found.
	virtual bool next();

//! Implementation
private:
	std::string path_with_ending_slash(const std::string &path);

	WIN32_FIND_DATA fileinfo;

	HANDLE handle;

	bool first_next;

	std::string scan_exp;

	std::string directory_path;
};

#endif
