/*
	$Id: rect.cpp,v 1.8 2003/02/04 09:03:14 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "Core/precomp.h"
#include "API/Core/Math/rect.h"

#include <cmath>

using namespace std;

CL_Rect CL_Rect::get_rot_bounds(const CL_Point &hotspot, float angle) const
{
	//Find the rotated positions of each corner
	CL_Rect retVal(*this);
	CL_Point ul = CL_Point(retVal.left, retVal.top).rotate(hotspot, angle);
	CL_Point ur = CL_Point(retVal.right, retVal.top).rotate(hotspot, angle);
	CL_Point ll = CL_Point(retVal.left, retVal.bottom).rotate(hotspot, angle);
	CL_Point lr = CL_Point(retVal.right, retVal.bottom).rotate(hotspot, angle);
	
	//Use the sidemost corners as the bounds of the new rectangle
	retVal.left = min(min(ul.x, ur.x), min(ll.x, lr.x));
	retVal.right = max(max(ul.x, ur.x), max(ll.x, lr.x));
	retVal.top = min(min(ul.y, ur.y), min(ll.y, lr.y));
	retVal.bottom = max(max(ul.y, ur.y), max(ll.y, lr.y));
	
	return retVal;
}

CL_Rect CL_Rect::get_rot_bounds(CL_Origin origin, int x, int y, float angle) const
{
	return get_rot_bounds(
		CL_Point(left, top) + calc_origin(origin, get_size()) + CL_Point(x, y),
		angle);
}

CL_Rectf CL_Rectf::get_rot_bounds(const CL_Pointf &hotspot, float angle) const
{
	//Find the rotated positions of each corner
	CL_Rectf retVal(*this);
	CL_Pointf ul = CL_Pointf(retVal.left, retVal.top).rotate(hotspot, angle);
	CL_Pointf ur = CL_Pointf(retVal.right, retVal.top).rotate(hotspot, angle);
	CL_Pointf ll = CL_Pointf(retVal.left, retVal.bottom).rotate(hotspot, angle);
	CL_Pointf lr = CL_Pointf(retVal.right, retVal.bottom).rotate(hotspot, angle);
	
	//Use the sidemost corners as the bounds of the new rectangle
	retVal.left = min(min(ul.x, ur.x), min(ll.x, lr.x));
	retVal.right = max(max(ul.x, ur.x), max(ll.x, lr.x));
	retVal.top = min(min(ul.y, ur.y), min(ll.y, lr.y));
	retVal.bottom = max(max(ul.y, ur.y), max(ll.y, lr.y));
	
	return retVal;
}

CL_Rectf CL_Rectf::get_rot_bounds(CL_Origin origin, float x, float y, float angle) const
{
	return get_rot_bounds(
		CL_Pointf(left, top) + calc_origin(origin, get_size()) + CL_Pointf(x, y),
		angle);
}
