
#include "Display/display_precomp.h"
#include "back_buffer_dx7.h"
#include "API/Core/System/error.h"

/////////////////////////////////////////////////////////////////////////////
// CL_BackBuffer_DX7 construction:

CL_BackBuffer_DX7::CL_BackBuffer_DX7(CL_DisplayWindow_DX7 *window) :
	window(window), data(0)
{
	// Initialize the pixel format structs.
	perform_lock();
	perform_unlock();
}

CL_BackBuffer_DX7::~CL_BackBuffer_DX7()
{
}

/////////////////////////////////////////////////////////////////////////////
// CL_BackBuffer_DX7 attributes:

LPDIRECTDRAWSURFACE CL_BackBuffer_DX7::get_surface()
{
	return window->get_backbuffer_surface();
}

/////////////////////////////////////////////////////////////////////////////
// CL_BackBuffer_DX7 operations:

void *CL_BackBuffer_DX7::get_data()
{
	return data;
}

void CL_BackBuffer_DX7::perform_lock()
{
	DDSURFACEDESC surface_desc;
	memset(&surface_desc, 0, sizeof(DDSURFACEDESC));
	surface_desc.dwSize = sizeof(DDSURFACEDESC);

	HRESULT err = get_surface()->Lock(
		NULL,
		&surface_desc,
		DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,
		NULL);

	if (err != DD_OK) { throw CL_Error("Lock failed"); }

	data = surface_desc.lpSurface;
	pitch = surface_desc.lPitch;
	width = surface_desc.dwWidth;
	height = surface_desc.dwHeight;
	format.set_depth(surface_desc.ddpfPixelFormat.dwRGBBitCount);
	format.set_red_mask(surface_desc.ddpfPixelFormat.dwRBitMask);
	format.set_green_mask(surface_desc.ddpfPixelFormat.dwGBitMask);
	format.set_blue_mask(surface_desc.ddpfPixelFormat.dwBBitMask);
	format.set_alpha_mask(surface_desc.ddpfPixelFormat.dwRGBAlphaBitMask);
}

void CL_BackBuffer_DX7::perform_unlock()
{
	get_surface()->Unlock(data);
}

/////////////////////////////////////////////////////////////////////////////
// CL_BackBuffer_DX7 implementation:
