
#ifndef header_display_window_dx7
#define header_display_window_dx7

#include "Display/Win32/display_window_win32.h"
#include "API/Display/graphic_context.h"
#include <ddraw.h>

class CL_DisplayWindow_DX7 : public CL_DisplayWindow_Win32
{
//! Construction:
public:
	CL_DisplayWindow_DX7();

	virtual ~CL_DisplayWindow_DX7();

//! Attributes:
public:
	virtual CL_GraphicContext *get_gc();

	LPDIRECTDRAW get_directdraw() { return directdraw; }

	const DDCAPS &get_hel_caps() const { return hel_caps; }
	
	const DDCAPS &get_hal_caps() const { return hal_caps; }

	LPDIRECTDRAWSURFACE get_primary_surface() { return primary_surface; }

	LPDIRECTDRAWSURFACE get_backbuffer_surface() { return backbuffer_surface; }

	virtual CL_PixelBuffer &get_buffer(int i);

	virtual const CL_PixelBuffer &get_buffer(int i) const;

	virtual int get_buffer_count() const;

//! Operations:
public:
	virtual void create_window(const CL_DisplayWindowDescription &desc);

	virtual void destroy_window();

	//: Flip buffers.
	virtual void flip();

//! Implementation:
private:
	void create_fullscreen_targets();

	void create_windowed_targets();
	
	void create_windowed_backbuffer();

	LPDIRECTDRAW directdraw;

	DDCAPS hel_caps;
	
	DDCAPS hal_caps;

	LPDIRECTDRAWSURFACE primary_surface;

	LPDIRECTDRAWSURFACE backbuffer_surface;

	int backbuffer_width, backbuffer_height;

	bool video_memory;

	bool can_flip;

	CL_PixelBuffer *pixel_buffers[2];

	CL_GraphicContext gc;
};

#endif
