
#include "surface_dx7.h"
#include "API/Display/pixel_buffer.h"
#include "API/Display/display.h"
#include "API/Display/display_window.h"
#include "API/Display/pixel_format.h"

/////////////////////////////////////////////////////////////////////////////
// CL_Surface_DX7 construction:

CL_Surface_DX7::CL_Surface_DX7(
	CL_PixelBuffer *provider,
	bool delete_provider,
	CL_Surface::Hint hint) : surface_buffer(0)
{
	CL_PixelFormat format = CL_Display::get_current_window()->get_back_buffer().get_format();

	provider->lock();
	try
	{
		surface_buffer = new CL_PixelBuffer(
			provider->get_width(),
			provider->get_height(),
			provider->get_width()*4,
			format);

		provider->convert(surface_buffer);
	}
	catch(...)
	{
		provider->unlock();
		throw;
	}
	provider->unlock();
}

CL_Surface_DX7::~CL_Surface_DX7()
{
	delete surface_buffer;
}

/////////////////////////////////////////////////////////////////////////////
// CL_Surface_DX7 attributes:

int CL_Surface_DX7::get_width() const
{
	return surface_buffer->get_width();
}

int CL_Surface_DX7::get_height() const
{
	return surface_buffer->get_height();
}

/////////////////////////////////////////////////////////////////////////////
// CL_Surface_DX7 operations:

void CL_Surface_DX7::draw(
	CL_Surface_Generic *attributes,
	int x, int y,
	CL_GraphicContext *gc)
{
	if (gc == 0) gc = CL_Display::get_current_window()->get_gc();
	CL_PixelBuffer &backbuffer = CL_Display::get_current_window()->get_back_buffer();

	CL_Rect src(0, 0, surface_buffer->get_width(), surface_buffer->get_height());
	CL_Rect dest(x, y, x + src.get_width(), y + src.get_height());

//	surface_buffer->blit(&backbuffer, src, dest, blend_one_minus_src_alpha);
}

void CL_Surface_DX7::draw(
	CL_Surface_Generic *attributes,
	const CL_Rect &dest,
	CL_GraphicContext *gc)
{
	if (gc == 0) gc = CL_Display::get_current_window()->get_gc();
	CL_PixelBuffer &backbuffer = CL_Display::get_current_window()->get_back_buffer();

	CL_Rect src(0, 0, surface_buffer->get_width(), surface_buffer->get_height());
//	surface_buffer->blit(&backbuffer, src, dest, blend_one_minus_src_alpha);
}

void CL_Surface_DX7::draw(
	CL_Surface_Generic *attributes,
	const CL_Rect &src,
	const CL_Rect &dest,
	CL_GraphicContext *gc)
{
	if (gc == 0) gc = CL_Display::get_current_window()->get_gc();
	CL_PixelBuffer &backbuffer = CL_Display::get_current_window()->get_back_buffer();

//	surface_buffer->blit(&backbuffer, src, dest, blend_one_minus_src_alpha);
}

/////////////////////////////////////////////////////////////////////////////
// CL_Surface_DX7 implementation:
