
#include "Display/display_precomp.h"
#include "display_window_dx8.h"
#include "API/Core/System/error.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_DisplayWindow_DX8::CL_DisplayWindow_DX8() : direct3d8(0), device(0)
{
	direct3d8 = Direct3DCreate8(D3D_SDK_VERSION);
	if (direct3d8 == 0) throw CL_Error("Direct3DCreate8 failed!");
}

CL_DisplayWindow_DX8::~CL_DisplayWindow_DX8()
{
	if (device) device->Release();
	if (direct3d8) direct3d8->Release();
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

CL_GraphicContext *CL_DisplayWindow_DX8::get_gc()
{
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_DisplayWindow_DX8::create_window(const CL_DisplayWindowDescription &desc)
{
	D3DDEVTYPE device_type;
	DWORD behavior_flags;
	D3DPRESENT_PARAMETERS present_parameters;

	// todo: fill present parameters, devtype and behavior flags

	create_window(desc, device_type, behavior_flags, &present_parameters);
}

void CL_DisplayWindow_DX8::create_window(
	const CL_DisplayWindowDescription &desc,
	D3DDEVTYPE device_type,
	DWORD behavior_flags,
	D3DPRESENT_PARAMETERS *present_parameters)
{
	CL_DisplayWindow_Win32::create_window(desc);

	HRESULT result = direct3d8->CreateDevice(D3DADAPTER_DEFAULT, device_type, get_hwnd(), behavior_flags, present_parameters, &device);
	if (FAILED(result)) throw CL_Error("CreateDevice failed!");
}

void CL_DisplayWindow_DX8::destroy_window()
{
	if (device) device->Release();
	device = 0;

	CL_DisplayWindow_Win32::destroy_window();
}

/////////////////////////////////////////////////////////////////////////////
// Implementation:
