
#include "Display/display_precomp.h"
#include "API/Display/Providers/provider_factory.h"
#include "API/Display/Providers/provider_type.h"
#include "API/Core/System/error.h"
#include "API/Core/System/clanstring.h"
#include <iostream>

/////////////////////////////////////////////////////////////////////////////
// CL_ProviderFactory attributes:

std::map<std::string, CL_ProviderType *> CL_ProviderFactory::types;

/////////////////////////////////////////////////////////////////////////////
// CL_ProviderFactory operations:

CL_PixelBuffer *CL_ProviderFactory::load(
	const std::string &filename,
	const std::string &type,
	CL_InputSourceProvider *input_provider)
{
	if (type != "")
	{
		if (types.find(type) == types.end()) throw CL_Error("Unknown image provider type " + type);

		CL_ProviderType *factory = types[type];
		return factory->load(filename, input_provider);
	}

	// Determine file extension and use it to lookup type.
	std::string ext = CL_String::right(filename, 4);
	if (ext.at(0) == '.') ext = CL_String::right(ext, 3);
	ext = CL_String::to_lower(ext);
	if (types.find(ext) == types.end()) throw CL_Error(std::string("Unknown image provider type ") + ext);

	CL_ProviderType *factory = types[ext];
	return factory->load(filename, input_provider);
}
