
#ifndef header_display_window_win32
#define header_display_window_win32

#include "Display/display_window_generic.h"
#include "API/Core/Math/rect.h"
#include "API/Display/input_context.h"
#include "API/Display/input_device.h"
#include <map>

class CL_DisplayWindow_Win32 : public CL_DisplayWindow_Generic
{
//! Construction:
public:
	CL_DisplayWindow_Win32();

	virtual ~CL_DisplayWindow_Win32();

//! Attributes:
public:
	virtual int get_width() const;

	virtual int get_height() const;

	virtual bool is_fullscreen() const;

	virtual bool has_focus() const;

	virtual CL_PixelBuffer &get_buffer(int i);

	virtual const CL_PixelBuffer &get_buffer(int i) const;

	virtual int get_buffer_count() const;

	virtual bool get_keycode(int keycode) const;

	HWND get_hwnd() { return hwnd; }

	virtual CL_InputContext *get_ic() { return &input_context; }

	const CL_Rect &get_layer_position() const { return layered_position; }

//! Operations:
public:
	virtual void set_fullscreen(int width, int height, int bpp);

	virtual void set_windowed();

	virtual void set_title(const std::string &title);

	virtual void set_position(const CL_Rect &pos);

	virtual void set_position(int x, int y);

	virtual void set_size(int width, int height);

	virtual void set_buffer_count(int flipping_buffers);

	virtual void update(const CL_Rect &rect);

	virtual void flip();

	virtual void create_window(const CL_DisplayWindowDescription &desc);

	virtual void destroy_window();

	virtual void show_system_cursor();

	virtual void hide_system_cursor();

	void set_videomode(int width, int height, int bpp);

	void restore_videomode();

	virtual bool received_event(UINT uMsg, WPARAM wParam, LPARAM lParam);

//! Implementation:
private:
	void received_keyboard_input(UINT uMsg, WPARAM wParam, LPARAM lParam);

	void received_mouse_input(UINT uMsg, WPARAM wParam, LPARAM lParam);

	void received_mouse_move(UINT uMsg, WPARAM wParam, LPARAM lParam);

	static std::map<HWND, CL_DisplayWindow_Win32 *> window_map;

	static LONG WINAPI message_handler(
		HWND hWnd,
		UINT uMsg,
		WPARAM wParam,
		LPARAM lParam);

	CL_InputContext input_context;

	CL_InputDevice keyboard;

	CL_InputDevice mouse;

	HWND hwnd;

	std::string title;

	bool fullscreen;

	bool allow_resize;

	bool layered;

	CL_Rect layered_position;

	int bpp;

	unsigned char keys_down[256];

	CL_Rect saved_position;

	CL_Point mouse_pos;

	bool left_ctrl_down, left_alt_down, left_shift_down;
	bool right_ctrl_down, right_alt_down, right_shift_down;
};

#endif
