
#include "Display/display_precomp.h"
#include "API/Display/input_event.h"
#include "input_device_win32mouse.h"
#include <stdio.h>

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice_Win32Mouse construction:

CL_InputDevice_Win32Mouse::CL_InputDevice_Win32Mouse(HWND hwnd) : hwnd(hwnd)
{
	type = CL_InputDevice::mouse;
	for (int i=0; i<32; i++) key_states[i] = false;
	slots.connect(sig_key_down, this, &CL_InputDevice_Win32Mouse::on_key_down);
	slots.connect(sig_key_up, this, &CL_InputDevice_Win32Mouse::on_key_up);
}

CL_InputDevice_Win32Mouse::~CL_InputDevice_Win32Mouse()
{
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice_Win32Mouse attributes:

int CL_InputDevice_Win32Mouse::get_x() const
{
	POINT cursor_pos;
	GetCursorPos(&cursor_pos);

	BOOL res = ScreenToClient(hwnd, &cursor_pos);
	if (res == FALSE) return 0;

	return cursor_pos.x;
}

int CL_InputDevice_Win32Mouse::get_y() const
{
	POINT cursor_pos;
	GetCursorPos(&cursor_pos);

	BOOL res = ScreenToClient(hwnd, &cursor_pos);
	if (res == FALSE) return 0;

	return cursor_pos.y;
}

bool CL_InputDevice_Win32Mouse::get_keycode(int keycode) const
{
	if (keycode < 0 || keycode >= 32) return false;
	return key_states[keycode];
}

std::string CL_InputDevice_Win32Mouse::get_key_name(int id) const
{
	switch (id)
	{
	case 0: return "Mouse left";
	case 1: return "Mouse right";
	case 2: return "Mouse middle";
	case 3: return "Mouse wheel up";
	case 4: return "Mouse wheel down";
	}

	char buf[256];
	sprintf(buf, "Mouse button %d", id);
	return buf;
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice_Win32Mouse operations:

void CL_InputDevice_Win32Mouse::set_position(int x, int y)
{
	POINT pt;
	pt.x = x;
	pt.y = y;
	
	ClientToScreen(hwnd, &pt);
	SetCursorPos(pt.x, pt.y);
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice_Win32Mouse implementation:

void CL_InputDevice_Win32Mouse::on_key_down(const CL_InputEvent &event)
{
	key_states[event.id] = true;
}

void CL_InputDevice_Win32Mouse::on_key_up(const CL_InputEvent &event)
{
	key_states[event.id] = false;
}
