
#ifndef header_input_device_x11mouse
#define header_input_device_x11mouse

#include "../input_device_generic.h"
#include "API/signals.h"
#include "API/Core/Math/point.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>

class CL_DisplayWindow_OpenGL;

class CL_InputDevice_X11Mouse : public CL_InputDevice_Generic
{
//!Construction:
public:
	CL_InputDevice_X11Mouse(CL_DisplayWindow_OpenGL *owner);

	virtual ~CL_InputDevice_X11Mouse();

//!Attributes:
public:
	//: Returns the x position of the device.
	virtual int get_x() const;

	//: Returns the y position of the device.
	virtual int get_y() const;

	//: Returns true if the passed key code is down for this device.
	virtual bool get_keycode(int keycode) const;

//!Operations:
public:
	//: Key name for specified identifier (A, B, C, Space, Enter, Backspace).
	virtual std::string get_key_name(int id) const;

//!Implementation:
private:
	void on_xevent(XEvent &event);
	
	void received_mouse_input(XEvent &event);
	void received_mouse_move(XEvent &event);
	
	CL_Slot slot_xevent;
	
	CL_Point mouse_pos;
	Time time_at_last_press;
	Time dblclk_time;
	int last_press_id;
	
	CL_DisplayWindow_OpenGL *owner;
};

#endif
