
#include "Display/display_precomp.h"
#include "API/Display/canvas.h"
#include "API/Display/display.h"
#include "API/Display/display_window.h"
#include "canvas_generic.h"
#include "display_target.h"

/////////////////////////////////////////////////////////////////////////////
// CL_Canvas construction:

CL_Canvas::CL_Canvas(
	const CL_Size &size,
	Format format,
	bool use_colorkey,
	unsigned int colorkey)
: impl(0)
{
	impl = CL_DisplayTarget::current->create_canvas(size, format, use_colorkey, colorkey);
}

CL_Canvas::CL_Canvas(
	CL_PixelBuffer *provider,
	bool delete_provider,
	Format format,
	bool use_colorkey,
	unsigned int colorkey)
: impl(0)
{
	impl = CL_DisplayTarget::current->create_canvas(provider, delete_provider, format, use_colorkey, colorkey);
}

CL_Canvas::CL_Canvas(const CL_Canvas &copy)
: impl(copy.impl)
{
	if (impl) impl->add_ref();
}

CL_Canvas::CL_Canvas()
: impl(0)
{
}

CL_Canvas::~CL_Canvas()
{
	if (impl) impl->release_ref();
}

/////////////////////////////////////////////////////////////////////////////
// CL_Canvas attributes:

CL_PixelBuffer &CL_Canvas::get_pixelbuffer()
{
	return impl->get_pixelbuffer();
}

CL_GraphicContext *CL_Canvas::get_gc()
{
	return impl->get_gc();
}

/////////////////////////////////////////////////////////////////////////////
// CL_Canvas operations:

CL_Canvas &CL_Canvas::operator =(const CL_Canvas &copy)
{
	if (impl) impl->release_ref();
	impl = copy.impl;
	if (impl) impl->add_ref();
	return *this;
}

void CL_Canvas::draw(int x, int y, CL_GraphicContext *gc)
{
	if (gc == 0) gc = CL_Display::get_current_window()->get_gc();
	impl->draw(x, y, gc);
}

/////////////////////////////////////////////////////////////////////////////
// CL_Canvas implementation:
