
#include "Display/display_precomp.h"
#include "API/Display/input_device.h"
#include "input_device_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice construction:

CL_InputDevice::CL_InputDevice()
: impl(0)
{
}

CL_InputDevice::CL_InputDevice(const CL_InputDevice &copy)
: impl(copy.impl)
{
	if (impl) impl->add_ref();
}

CL_InputDevice::~CL_InputDevice()
{
	if (impl) impl->release_ref();
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice attributes:

CL_InputDevice::Type CL_InputDevice::get_type() const
{
	return impl->type;
}

std::string CL_InputDevice::get_key_name(int id) const
{
	return impl->get_key_name(id);
}

int CL_InputDevice::get_x() const
{
	return impl->get_x();
}

int CL_InputDevice::get_y() const
{
	return impl->get_y();
}

bool CL_InputDevice::get_keycode(int keycode) const
{
	return impl->get_keycode(keycode);
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice operations:

CL_InputDevice &CL_InputDevice::operator =(const CL_InputDevice &copy)
{
	if (impl) impl->release_ref();
	impl = copy.impl;
	if (impl) impl->add_ref();

	return *this;
}

void CL_InputDevice::set_position(int x, int y)
{
	impl->set_position(x, y);
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice signals:

CL_Signal_v1<const CL_InputEvent &> &CL_InputDevice::sig_key_down()
{
	return impl->sig_key_down;
}

CL_Signal_v1<const CL_InputEvent &> &CL_InputDevice::sig_key_up()
{
	return impl->sig_key_up;
}

CL_Signal_v1<const CL_InputEvent &> &CL_InputDevice::sig_key_dblclk()
{
	return impl->sig_key_dblclk;
}

CL_Signal_v1<const CL_InputEvent &> &CL_InputDevice::sig_move()
{
	return impl->sig_move;
}

/////////////////////////////////////////////////////////////////////////////
// CL_InputDevice implementation:

CL_InputDevice::CL_InputDevice(CL_InputDevice_Generic *impl)
: impl(impl)
{
	if (impl) impl->add_ref();
}
