
#ifndef header_input_device_generic
#define header_input_device_generic

#include "API/Display/input_device.h"

class CL_InputDevice_Generic
{
//! Construction:
public:
	CL_InputDevice_Generic();

	virtual ~CL_InputDevice_Generic();

//! Attributes:
public:
	//: The input device type.
	CL_InputDevice::Type type;

	//: Returns the x position of the device.
	virtual int get_x() const { return 0; }

	//: Returns the y position of the device.
	virtual int get_y() const { return 0; }

	//: Returns true if the passed key code is down for this device.
	virtual bool get_keycode(int keycode) const { return false; }

	//: Key name for specified identifier (A, B, C, Space, Enter, Backspace).
	virtual std::string get_key_name(int id) const = 0;

//! Operations:
public:
	//: Sets the position of the device.
	//- <p>Only valid for mouse.</p>
	virtual void set_position(int x, int y) {};

	void add_ref();

	void release_ref();

//! Signals:
public:
	CL_Signal_v1<const CL_InputEvent &> sig_key_down;
	CL_Signal_v1<const CL_InputEvent &> sig_key_up;
	CL_Signal_v1<const CL_InputEvent &> sig_key_dblclk;
	CL_Signal_v1<const CL_InputEvent &> sig_move;

//! Implementation:
private:
	int ref_count;
};

#endif
