
#ifndef header_pixel_buffer_generic
#define header_pixel_buffer_generic

#include "API/Display/pixel_format.h"
#include "API/Display/palette.h"

//: Pixel data implementation interface.
class CL_PixelBuffer_Generic
{
//! Construction:
public:
	CL_PixelBuffer_Generic() : lock_count(0), ref_count(0) { return; }

	virtual ~CL_PixelBuffer_Generic() { return; }

//! Attributes:
public:
	//: Buffer pixel format description.
	CL_PixelFormat format;

	//: Palette, if 8 bpp pixel format.
	CL_Palette palette;

	//: Buffer scanline pitch.
	unsigned int pitch;

	//: Buffer width.
	int width;

	//: Buffer height.
	int height;

	//: Lock reference count.
	int lock_count;

//! Operations:
public:
	//: Returns a pointer to the beginning of the pixel buffer. Pointer
	//: is only valid inside a lock/unlock session.
	virtual void *get_data()=0;

	//: Reference counted lock function. Allocate pixel buffer in this function. 
	virtual void perform_lock()=0;

	//: Reference counted unlock function. Free pixel buffer in this function.
	virtual void perform_unlock()=0;

	int add_ref() { return ++ref_count; }

	int release_ref()
	{
		ref_count--;
		if (ref_count == 0) { delete this; return 0; }
		return ref_count;
	}

//! Implementation:
private:
	int ref_count;
};

#endif
