
#include "Display/display_precomp.h"
#include "resourcedata_font.h"
#include "API/Display/sprite.h"
#include "API/Core/System/clanstring.h"
#include "API/Core/System/error.h"
#include "API/Core/Resources/resourceoptions.h"
#include "API/Core/Resources/resource_manager.h"

//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Font construction:

CL_ResourceData_Font::CL_ResourceData_Font(CL_Resource &resource)
: CL_ResourceData(resource)
{
	slots.connect(resource.sig_load_file(), this, &CL_ResourceData_Font::on_load_file);
	slots.connect(resource.sig_load_datafile(), this, &CL_ResourceData_Font::on_load_datafile);
	slots.connect(resource.sig_save_datafile(), this, &CL_ResourceData_Font::on_save_datafile);
	slots.connect(resource.sig_unload(), this, &CL_ResourceData_Font::on_unload);

	resource.attach_data("font", this);
}
	
CL_ResourceData_Font::~CL_ResourceData_Font()
{
}
	
//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Font attributes:


//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Font implementation:

void CL_ResourceData_Font::on_load_file()
{
	CL_Resource resource = get_resource();
	CL_ResourceOptions &options = resource.get_options();
	CL_ResourceManager manager = resource.get_manager();
	
	//First check if it's a system font
	if (options.exists("font_name")) 
	{
		if (!options.exists("height"))
			throw CL_Error("System font resource has no 'height' variable.");
		
		const std::string font_name = options.get_option("font_name").get_value();
		const int height = atoi(options.get_option("height").get_value().c_str());
		int width = 0;

		bool bold = false;
		bool italic = false;
		bool underline = false;
		bool strikeout = false;
		
		if (options.exists("width"))
			width = atoi(options.get_option("width").get_value().c_str());
		
		if (options.exists("bold"))
			bold = (atoi(options.get_option("bold").get_value().c_str()) != 0);
		
		if (options.exists("italic"))
			italic = (atoi(options.get_option("italic").get_value().c_str()) !=0);
		
		if (options.exists("underline"))
			underline = (atoi(options.get_option("underline").get_value().c_str()) != 0);
		
		if (options.exists("strikeout"))
			strikeout = (atoi(options.get_option("strikeout").get_value().c_str()) != 0);
		
		if (options.exists("letters"))
		{
			font = CL_Font(
				font_name,
				options.get_option("letters").get_value(),
				height,
				width,
				bold,
				italic,
				underline,
				strikeout);
		}
		else
		{
			font = CL_Font(
				font_name,
				height,
				width,
				bold,
				italic,
				underline,
				strikeout);
		}
	} 
	else
	{
		if (!options.exists("glyphs")) 
			throw CL_Error("Font resource has no 'glyphs' variable.");
		
		if (!options.exists("letters")) 
			throw CL_Error("Font resource has no 'letters' variable.");
		
		//Set most values to CL_Font defaults, then we can override them with options if they exist
		const CL_Sprite spr_glyphs(options.get_option("glyphs").get_value(), &manager);
		const std::string letters = options.get_option("letters").get_value();
		int spacelen = -1;
		int width_offset = 0;
		int height_offset = 0;
		int baseline = 0;
		
		if (options.exists("spacelen"))
			spacelen = atoi(options.get_option("spacelen").get_value().c_str());
		
		if (options.exists("width_offset"))
			width_offset = atoi(options.get_option("width_offset").get_value().c_str());
		
		if (options.exists("height_offset"))
			height_offset = atoi(options.get_option("height_offset").get_value().c_str());
		
		if (options.exists("baseline"))
			baseline = atoi(options.get_option("baseline").get_value().c_str());
		
		font = CL_Font(
			spr_glyphs,
			letters,
			spacelen,
			width_offset,
			height_offset,
			baseline);
	}
	
	font.resource = resource;
}
	
void CL_ResourceData_Font::on_load_datafile(CL_InputSourceProvider *)
{
}
	
void CL_ResourceData_Font::on_save_datafile(CL_OutputSourceProvider *)
{
}
	
void CL_ResourceData_Font::on_unload()
{
	font = CL_Font();
}
