
#include "Display/display_precomp.h"
#include "resourcedata_sprite_description.h"
#include "API/Display/sprite_description.h"
#include "API/Display/pixel_buffer.h"
#include "API/Core/System/clanstring.h"
#include "API/Core/System/error.h"
#include "API/Core/Resources/resourceoptions.h"
#include "API/Core/Resources/resource_manager.h"
#include "API/Display/Providers/provider_factory.h"

#include <iostream>

//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_SpriteDescription construction:

CL_ResourceData_SpriteDescription::CL_ResourceData_SpriteDescription(CL_Resource &resource, const std::string &type)
: CL_ResourceData(resource)
{
	resource.attach_data(type, this);
}
	
CL_ResourceData_SpriteDescription::~CL_ResourceData_SpriteDescription()
{
}
	
//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_SpriteDescription attributes:


//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_SpriteDescription implementation:

CL_SpriteDescription CL_ResourceData_SpriteDescription::load_description()
{
	CL_Resource resource = get_resource();
	CL_ResourceOptions &options = resource.get_options();
	
	CL_SpriteDescription desc;
	
	int cur_image = 1;
	while (true)
	{
		std::string image_var = "image";
		image_var += CL_String::from_int(cur_image);
		if (!options.exists(image_var)) break;

		bool has_method = options.exists(image_var + "_method");
		bool has_pos = options.exists(image_var + "_pos");
		bool has_size = options.exists(image_var + "_size");
		bool has_array = options.exists(image_var + "_array");
		bool has_array_skipframes = options.exists(image_var + "_array_skipframes");
		bool has_spacing = options.exists(image_var + "_spacing");
		bool has_tcol = options.exists(image_var + "_tcol");
		bool has_trans_limit = options.exists(image_var + "_trans_limit");
		
		std::string image_name = options.get_option(image_var).get_value();
		CL_PixelBuffer *image = load_image(image_name);

		try
		{
			std::string method;

			if (has_method)
			{
				method = options.get_option(image_var + "_method").get_value();
			}
			else
			{
//				std::cout << "No sprite imageX_method type - assumed method [";
				if(has_trans_limit)
					method = "alpha";
				else if(has_array)
					method = "grid";
				else
					method = "single";
//				std::cout << method << "]" << std::endl;
			}

			if (method == "none" || method == "single")
			{
				desc.add_frame(image, true);
			}
			else if (method == "grid")
			{
				if (!has_size) throw CL_Error("Resource is missing imageX_size attribute.");

				int xpos = 0;
				int ypos = 0;
				int xarray = 1;
				int yarray = 1;
				int array_skipframes = 0;
				int xspacing = 0;
				int yspacing = 0;

				int width = atoi(options.get_option(image_var + "_size").get_value(0).c_str());
				int height = atoi(options.get_option(image_var + "_size").get_value(1).c_str());
				
				if (has_pos)
				{
					xpos = atoi(options.get_option(image_var + "_pos").get_value(0).c_str());
					ypos = atoi(options.get_option(image_var + "_pos").get_value(1).c_str());
				}
				
				if (has_array)
				{
					xarray = atoi(options.get_option(image_var + "_array").get_value(0).c_str());
					yarray = atoi(options.get_option(image_var + "_array").get_value(1).c_str());
				}
				if (has_array_skipframes)
				{
					array_skipframes = atoi(options.get_option(image_var + "_array_skipframes").get_value().c_str());
				}

				if (has_spacing)
				{
					xspacing = atoi(options.get_option(image_var + "_spacing").get_value(0).c_str());
					yspacing = atoi(options.get_option(image_var + "_spacing").get_value(1).c_str());
				}
				
				desc.add_gridclipped_frames(
					image,
					xpos, ypos,
					width, height,
					xarray, yarray,
					array_skipframes,
					xspacing, yspacing,
					true);
			}
			else if (method == "palette")
			{
				int xpos = 0;
				int ypos = 0;

				if (has_pos)
				{
					xpos = atoi(options.get_option(image_var + "_pos").get_value(0).c_str());
					ypos = atoi(options.get_option(image_var + "_pos").get_value(1).c_str());
				}
				
				desc.add_paletteclipped_frames(
					image,
					xpos, ypos,
					true);
			}
			else if (method == "alpha")
			{
				int xpos = 0;
				int ypos = 0;
				float trans_limit = 0.05f;

				if (has_pos)
				{
					xpos = atoi(options.get_option(image_var + "_pos").get_value(0).c_str());
					ypos = atoi(options.get_option(image_var + "_pos").get_value(1).c_str());
				}
				
				if (has_trans_limit)
				{
					trans_limit = (float)atof(options.get_option(image_var + "_trans_limit").get_value().c_str());
				}

				desc.add_alphaclipped_frames(
					image,
					xpos, ypos,
					trans_limit,
					true);
			}
			else
			{
				throw CL_Error("Unknown sprite imageX_method type!");
			}
		}
		catch (...)
		{
//			delete image;
			throw;
		}

		cur_image++;
	}
	
	if (cur_image == 1) throw CL_Error("Sprite resource contained no frames!");

	return desc;
}
	
CL_PixelBuffer *CL_ResourceData_SpriteDescription::load_image(const std::string &filename)
{
	return CL_ProviderFactory::load(filename, "", get_resource().get_manager().get_resource_provider());
}
