
#include "Display/display_precomp.h"
#include "API/Display/setupdisplay.h"
#include "API/Display/Providers/provider_type_register.h"
#include "API/Display/Providers/pcx_provider.h"
#include "API/Display/Providers/targa_provider.h"
#include "API/Display/Providers/jpeg_provider.h"
#include "API/Display/Providers/png_provider.h"
#include "API/Core/Resources/resourcetype.h"
#include "resourcedata_surface.h"
#include "resourcedata_sprite.h"
#include "resourcedata_sprite_description.h"
#include "resourcedata_texture.h"
#include "resourcedata_font.h"

#include <Hermes/Hermes.h>

static std::list<CL_ResourceType *> resource_types;
static int init_count = 0;

void CL_SetupDisplay::init(bool register_resources_only)
{
	init_count++;
	if (init_count != 1) return;

	resource_types.push_back(new CL_RegisterResourceType<CL_ResourceData_Surface>("surface"));
	resource_types.push_back(new CL_RegisterResourceType<CL_ResourceData_Sprite>("sprite"));
	resource_types.push_back(new CL_RegisterResourceType<CL_ResourceData_Sprite>("description"));
	resource_types.push_back(new CL_RegisterResourceType<CL_ResourceData_Font>("font"));
	resource_types.push_back(new CL_RegisterResourceType<CL_ResourceData_Texture>("texture"));

	new CL_ProviderType_Register<CL_PCXProvider>("pcx");
	new CL_ProviderType_Register<CL_TargaProvider>("tga");
	new CL_ProviderType_Register<CL_TargaProvider>("targa");
	new CL_ProviderType_Register<CL_JPEGProvider>("jpeg");
	new CL_ProviderType_Register<CL_JPEGProvider>("jpg");
	new CL_ProviderType_Register<CL_PNGProvider>("png");
	
	if (register_resources_only) return;

	Hermes_Init();
}

void CL_SetupDisplay::deinit()
{
	init_count--;
	if (init_count != 0) return;
	
	std::list<CL_ResourceType *>::iterator it;
	for (it = resource_types.begin(); it != resource_types.end(); ++it)
	{
		delete *it;
	}
	resource_types = std::list<CL_ResourceType *>();
	
	Hermes_Done();
}
