
#include "Display/display_precomp.h"
#include "API/Display/texture.h"
#include "API/Display/display.h"
#include "API/Display/display_window.h"
#include "API/Display/Providers/provider_factory.h"
#include "API/Core/System/error.h"
#include "API/Core/Resources/resource_manager.h"
#include "texture_generic.h"
#include "display_target.h"
#include "resourcedata_texture.h"

//////////////////////////////////////////////////////////////////////////////
// CL_Texture construction:

CL_Texture::CL_Texture()
: impl(0)
{
}

CL_Texture::CL_Texture(
	CL_PixelBuffer *provider,
	bool delete_provider,
	Hint hint)
: impl(0)
{
	impl = CL_DisplayTarget::current->create_texture(provider, delete_provider);
	impl->add_ref();
}

CL_Texture::CL_Texture(
	const std::string &resource_id,
	CL_ResourceManager *manager)
: impl(0)
{
	CL_Resource &resource = manager->get_resource(resource_id);
	resource.load();

	CL_ResourceData_Texture *data =
		(CL_ResourceData_Texture *) resource.get_data("surface");

	*this = data->get_texture();
}

CL_Texture::CL_Texture(const std::string &filename, Hint hint)
: impl(0)
{
	*this = CL_Texture(CL_ProviderFactory::load(filename), true, hint);
}

CL_Texture::CL_Texture(const CL_Texture &copy)
: impl(copy.impl)
{
	if (impl) impl->add_ref();
	resource = copy.resource;
	resource.load();
}

CL_Texture::~CL_Texture()
{
	if (impl) impl->release_ref();
	resource.unload();
}

//////////////////////////////////////////////////////////////////////////////
// CL_Texture attributes:

int CL_Texture::get_width() const
{
	return impl->screen_size.width;
}
	
int CL_Texture::get_height() const
{
	return impl->screen_size.height;
}

int CL_Texture::get_texture_width() const
{
	return impl->texture_size.width;
}
	
int CL_Texture::get_texture_height() const
{
	return impl->texture_size.height;
}

//////////////////////////////////////////////////////////////////////////////
// CL_Texture operations:

CL_Texture &CL_Texture::operator =(const CL_Texture &copy)
{
	resource.unload();
	if (impl) impl->release_ref();
	impl = copy.impl;
	if (impl) impl->add_ref();
	resource = copy.resource;
	resource.load();
	return *this;
}

void CL_Texture::bind(CL_GraphicContext *gc)
{
	if (gc == 0) gc = CL_Display::get_current_window()->get_gc();
	if (impl) impl->bind(gc);
}

//////////////////////////////////////////////////////////////////////////////
// CL_Texture implementation:
