
#ifndef header_display_window_opengl
#define header_display_window_opengl

#include "Display/Win32/display_window_win32.h"
#include "API/Display/graphic_context.h"
#include "API/Display/pixel_buffer.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <stack>

class CL_DisplayWindow_OpenGL : public CL_DisplayWindow_Win32
{
//! Construction:
public:
	CL_DisplayWindow_OpenGL();

	virtual ~CL_DisplayWindow_OpenGL();

//! Attributes:
public:
	//: Returns the gc of this window.
	virtual CL_GraphicContext *get_gc();

	//: Returns pixel buffer of frame buffer.
	virtual CL_PixelBuffer &get_buffer(int i);

	virtual const CL_PixelBuffer &get_buffer(int i) const;

	//: Returns the number of frame buffers.
	virtual int get_buffer_count() const;

	//: OpenGL rendering context for this window.
	HGLRC get_context() { return context; }

	//: Device context for this window.
	HDC get_dc();

//! Operations:
public:
	//: Called when a window is to be created.
	virtual void create_window(const CL_DisplayWindowDescription &desc);

	//: Called when a window is destroyed.
	virtual void destroy_window();

	//: Called when window receives a windows event.
	virtual bool received_event(UINT uMsg, WPARAM wParam, LPARAM lParam);

	//: Flip opengl buffers.
	virtual void flip();

	//: Select the opengl context for this window.
	virtual void select_context();

	//: Copy a region of the backbuffer to the frontbuffer.
	virtual void update(const CL_Rect &rect);

//! Implementation:
private:
	//: Repaint layered window.
	void repaint_layered();

	//: OpenGL compatible gc.
	CL_GraphicContext gc;

	//: OpenGL frame buffer access.
	CL_PixelBuffer buffer_front, buffer_back;

	//: OpenGL rendering context for this window.
	HGLRC context;

	//: Device context for this window.
	HDC hdc;

	//: Maximum allowed texture size for this graphic card.
	GLint max_texture_size;

	//: Device context to layered bitmap.
	HDC layered_dc;

	//: Layered bitmap.
	HBITMAP layered_bitmap;

	//: Original bitmap selected into the layered DC.
	HBITMAP layered_orig_bitmap;

	//: Layered window, user32.dll dll handle.
	HMODULE user32_dll;

	//: Layered window, function pointer for UpdateLayeredWindow.
	typedef BOOL (WINAPI *PtrUpdateLayeredWindow)(
	  HWND, HDC, POINT *, SIZE *, HDC, POINT *, COLORREF, BLENDFUNCTION *, DWORD);

	PtrUpdateLayeredWindow ptr_UpdateLayeredWindow;
};

#endif

