
#ifndef header_canvas_opengl
#define header_canvas_opengl

#include "Display/canvas_generic.h"
#include "Display/pixel_buffer_generic.h"
#include "API/Display/pixel_buffer.h"
#include "API/Display/graphic_context.h"

//: OpenGL canvas
class CL_Canvas_OpenGL : public CL_Canvas_Generic
{
//! Construction:
public:
	//: Construct empty canvas with the specified size and format.
	CL_Canvas_OpenGL(
		const CL_Size &size, CL_Canvas::Format format,
		bool use_colorkey, unsigned int colorkey);

	//: Construct canvas based on an existing pixel buffer.
	CL_Canvas_OpenGL(
		CL_PixelBuffer *provider, bool delete_provider,
		CL_Canvas::Format format,
		bool use_colorkey, unsigned int colorkey);

	virtual ~CL_Canvas_OpenGL();

//! Attributes:
public:
	//: Returns the pixel buffer of the canvas.
	virtual CL_PixelBuffer &get_pixelbuffer();

	//: Returns the graphic context of the canvas.
	virtual CL_GraphicContext *get_gc();

//! Operations:
public:
	//: Draw canvas onto screen.
	virtual void draw(int x, int y, CL_GraphicContext *gc);

	void select_context();

//! Implementation:
private:
	CL_Size texture_size;

	CL_Size canvas_size;

	unsigned int texture_handle;

	CL_PixelBuffer pixelbuffer;

	CL_GraphicContext *gc;
};

#endif
