
#ifndef header_gl_state_checker
#define header_gl_state_checker

#include <stack>
#include "gl_state.h"
#include <GL/gl.h>
#ifndef WIN32
#include <GL/glx.h>
#endif

class CL_Rectf;
class CL_Pointf;
class CL_Color;

class CL_GLStateChecker
{
//! Construction:
public:
	CL_GLStateChecker();

//! Operations:
public:
	//: Adds a clear screen.
	static void add_clear(
		const CL_Color &color);

	//: Adds drawal of a pixel to the state tree.
	static void add_pixel(
		const CL_GLState &state,
		const CL_Pointf &position);

	//: Adds drawal of a line to the state tree.
	static void add_line(
		const CL_GLState &state,
		const CL_Rectf &position);

	//: Adds drawal of a gradient to the state tree.
	static void add_gradient(
		const CL_GLState &state,
		const CL_Rectf &position,
		const CL_Gradient &gradient);

	//: Adds drawal of a texture to the state tree.
	static void add_quad(
		const CL_GLState &state,
		const CL_Rectf &position,
		const CL_Rectf &tex_coords);

	//: Renders state tree objects.
	static void flush();

#ifdef WIN32
	// Selects specified context into opengl state machine.
	static void set_context(HDC drawable, HGLRC context);
#else
	// Selects specified context into opengl state machine.
	static void set_context(Display *disp, GLXDrawable drawable, GLXContext context);
#endif

//! Implementation:
private:
	static float calc_rotate_x(const float &px, const float &py)
	{
		return 
			internal_state.rotation_hotspot.x +
			(px-internal_state.rotation_hotspot.x) * rotate_x.x +
			(py-internal_state.rotation_hotspot.y) * rotate_y.x;
	}

	static float calc_rotate_y(const float &px, const float &py)
	{
		return 
			internal_state.rotation_hotspot.y +
			(px-internal_state.rotation_hotspot.x) * rotate_x.y +
			(py-internal_state.rotation_hotspot.y) * rotate_y.y;
	}

	static void set_state(const CL_GLState &state, GLenum primitive);

	static CL_GLState internal_state;

	static GLenum begin_primitive;

	static CL_Pointf rotate_x, rotate_y;

#ifdef WIN32
	//: Active opengl drawable.
	static HDC cur_drawable;

	//: Active opengl context.
	static HGLRC cur_context;
#else
	//: Active opengl drawable.
	static GLXDrawable cur_drawable;

	//: Active opengl context.
	static GLXContext cur_context;
#endif
};

#endif
