
#include "Display/display_precomp.h"
#include "API/GL/opengl_window.h"
#include "API/Display/display_window_description.h"

#ifdef WIN32
#include "WGL/display_window_opengl.h"
#endif

#ifdef USE_X11
#include "GLX/display_window_opengl.h"
#endif

#define gl_impl ((CL_DisplayWindow_OpenGL *) impl)

/////////////////////////////////////////////////////////////////////////////
// CL_OpenGLWindow construction:

CL_OpenGLWindow::CL_OpenGLWindow() : CL_DisplayWindow()
{
}

CL_OpenGLWindow::CL_OpenGLWindow(const CL_OpenGLWindow &copy) : CL_DisplayWindow(copy)
{
}

CL_OpenGLWindow::CL_OpenGLWindow(
	const std::string &title,
	int width,
	int height,
	bool start_fullscreen,
	bool allow_resize,
	int flipping_buffers)
: CL_DisplayWindow(new CL_DisplayWindow_OpenGL)
{
	CL_DisplayWindowDescription desc;
	desc.set_title(title);
	desc.set_size(CL_Size(width, height));
	desc.set_fullscreen(start_fullscreen);
	desc.set_allow_resize(allow_resize);
	desc.set_flipping_buffers(flipping_buffers);

	gl_impl->create_window(desc);

	// Automatically map first created display window as 'CL_Display current'.
	if (CL_DisplayWindow_Generic::current == 0) CL_DisplayWindow_Generic::current = this;

	gl_impl->sig_resize(get_width(), get_height());
}

CL_OpenGLWindow::~CL_OpenGLWindow()
{
}

/////////////////////////////////////////////////////////////////////////////
// CL_OpenGLWindow attributes:

#ifdef WIN32

HGLRC CL_OpenGLWindow::get_context()
{
	return gl_impl->get_context();
}

HDC CL_OpenGLWindow::get_dc()
{
	return gl_impl->get_dc();
}

HWND CL_OpenGLWindow::get_wnd()
{
	return gl_impl->get_hwnd();
}

#endif

/////////////////////////////////////////////////////////////////////////////
// CL_OpenGLWindow operations:

void CL_OpenGLWindow::select_context()
{
	gl_impl->select_context();
}
