
#include "Display/display_precomp.h"
#include "pixel_buffer_opengl_texture.h"
#include "Display/graphic_context_generic.h"
#include "gl_state_checker.h"

/////////////////////////////////////////////////////////////////////////////
// CL_PixelBuffer_OpenGL_Texture construction:

CL_PixelBuffer_OpenGL_Texture::CL_PixelBuffer_OpenGL_Texture(
	int offset_x,
	int offset_y,
	int width,
	int height,
	GLuint handle,
	CL_GraphicContext *gc)
: offset_x(offset_x), offset_y(offset_y), handle(handle), gc(gc), data(0)
{
	CL_PixelBuffer_Generic::width = width;
	CL_PixelBuffer_Generic::height = height;

	format.set_depth(32);
	format.set_red_mask  (0x000000ff);
	format.set_green_mask(0x0000ff00);
	format.set_blue_mask (0x00ff0000);
	format.set_alpha_mask(0xff000000);
	format.enable_colorkey(false);

	// todo: Use GL_TEXTURE_INTERNAL_FORMAT to use the native format of texture instead.
	// todo: Take advantage of pbuffer extension where available.

	gc->impl->select_gc();
	glBindTexture(GL_TEXTURE_2D, handle);
	
	int internal_format = 0;

	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &texture_width);
	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &texture_height);
	glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_INTERNAL_FORMAT, &internal_format);
	
	GLint align;
	glGetIntegerv(GL_PACK_ALIGNMENT, &align);
	pitch = (texture_width*4+align-1)/align;
	pitch *= align;
}

CL_PixelBuffer_OpenGL_Texture::~CL_PixelBuffer_OpenGL_Texture()
{
	delete[] data;
}

/////////////////////////////////////////////////////////////////////////////
// CL_PixelBuffer_OpenGL_Texture operations:

void *CL_PixelBuffer_OpenGL_Texture::get_data()
{
	return data+offset_x*4+offset_y*pitch;
}

void CL_PixelBuffer_OpenGL_Texture::perform_lock()
{
	data = new char[pitch*texture_height];

	gc->impl->select_gc();
	CL_GLStateChecker::flush();
	glBindTexture(GL_TEXTURE_2D, handle);
	glGetTexImage(GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
}

void CL_PixelBuffer_OpenGL_Texture::perform_unlock()
{
	gc->impl->select_gc();
	CL_GLStateChecker::flush();

	glBindTexture(GL_TEXTURE_2D, handle);
	glTexImage2D(
		GL_TEXTURE_2D,          // target
		0,                      // level
		GL_RGBA,                // internalformat
		texture_width,          // width
		texture_height,         // height
		0,                      // border
		GL_RGBA,                // format
		GL_UNSIGNED_BYTE,       // type
		data);                  // texels

	// Texture mode parameters must be setup before creating textures!
	// Dunno if this is a driver bug in both nvidia and my laptops s3.
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE_EXT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE_EXT);

	delete[] data;
	data = 0;
}

/////////////////////////////////////////////////////////////////////////////
// CL_PixelBuffer_OpenGL_Texture Implementation:
