
#ifndef header_surface_opengl
#define header_surface_opengl

#include <GL/gl.h>

#include "Display/surface_generic.h"
#include "Display/surface_target.h"
#include "API/Display/surface.h"
#include "API/Core/Math/size.h"
#include "GL/gl_state_checker.h"

class CL_Surface_OpenGL : public CL_Surface_Target
{
//! Construction:
public:
	CL_Surface_OpenGL(
		CL_PixelBuffer *provider,
		bool delete_provider,
		CL_Surface::Hint hint);

	virtual ~CL_Surface_OpenGL();

//! Attributes:
public:
	//: Returns the width.
	virtual int get_width() const;
	
	//: Returns the height.
	virtual int get_height() const;

//! Operations:
public:
	//: Draw surface on screen.
	virtual void draw(
		CL_Surface_Generic *attributes,
		int x, int y,
		CL_GraphicContext *gc);

	//: Draw surface on screen.
	virtual void draw(
		CL_Surface_Generic *attributes,
		const CL_Rect &dest,
		CL_GraphicContext *gc);

	//: Draw surface on screen.
	virtual void draw(
		CL_Surface_Generic *attributes,
		const CL_Rect &src,
		const CL_Rect &dest,
		CL_GraphicContext *gc);

//! Implementation:
private:
	CL_Sizef screen_size;
	CL_Sizef texture_size;
		
	GLuint handle;
	CL_GLState state;
	
	// inlined this function for performance reasons.
	void init_attributes(CL_Surface_Generic *attributes, const CL_Sizef &size)
	{
		state.blendfunc[0] = attributes->blend_src;
		state.blendfunc[1] = attributes->blend_dest;
		state.color[0] = attributes->red;
		state.color[1] = attributes->green;
		state.color[2] = attributes->blue;
		state.color[3] = attributes->alpha;
		state.texture_handle = handle;

		if (attributes->angle != 0.0f)
		{
			// Calculate rotation hotspot
			state.rotation_hotspot = calc_hotspot(
				attributes->rotation_origin,
				attributes->rotation_hotspot,
				size);
		}
		
		state.rotate_angle = attributes->angle;
	}

	// inlined this function for performance reasons.
	CL_Pointf CL_Surface_OpenGL::calc_hotspot(CL_Origin origin, const CL_Pointf &hotspot, const CL_Sizef &size)
	{
		switch(origin)
		{
		case origin_top_left:
		default:
			return CL_Pointf(-hotspot.x, hotspot.y);
			break;
		case origin_top_center:
			return CL_Pointf(size.width / 2 - hotspot.x, -hotspot.y);
			break;
		case origin_top_right:
			return CL_Pointf(size.width - hotspot.x, -hotspot.y);
			break;
		case origin_center_left:
			return CL_Pointf(-hotspot.x, size.height / 2 - hotspot.y);
			break;
		case origin_center:
			return CL_Pointf(size.width / 2 - hotspot.x, size.height / 2 - hotspot.y);
			break;
		case origin_center_right:
			return CL_Pointf(size.width - hotspot.x, size.height / 2 - hotspot.y);
			break;
		case origin_bottom_left:
			return CL_Pointf(-hotspot.x, size.height - hotspot.y);
			break;
		case origin_bottom_center:
			return CL_Pointf(size.width / 2 - hotspot.x, size.height - hotspot.y);
			break;
		case origin_bottom_right:
			return CL_Pointf(size.width - hotspot.x, size.height - hotspot.y);
			break;
		}
	}
};

#endif
