
#include "Display/display_precomp.h"
#include "texture_opengl.h"
#include "API/Display/pixel_buffer.h"
#include "API/Display/graphic_context.h"
#include "API/Display/pixel_format.h"

//////////////////////////////////////////////////////////////////////////////
// CL_Texture_OpenGL construction:

CL_Texture_OpenGL::CL_Texture_OpenGL(
	CL_PixelBuffer *provider,
	bool delete_provider)
{
	// Convert provider to OpenGL texture format ABGR:
	CL_PixelFormat abgr_format;
	abgr_format.set_depth(32);
	abgr_format.enable_colorkey(false);
	abgr_format.set_alpha_mask(0xFF000000);
	abgr_format.set_blue_mask (0x00FF0000);
	abgr_format.set_green_mask(0x0000FF00);
	abgr_format.set_red_mask  (0x000000FF);

	provider->lock();

	screen_size.width = provider->get_width();
	screen_size.height = provider->get_height();
	
	texture_size.width = 1;
	texture_size.height = 1;
	while (texture_size.width < screen_size.width) texture_size.width *= 2;
	while (texture_size.height < screen_size.height) texture_size.height *= 2;

	CL_PixelBuffer *buffer = new CL_PixelBuffer(
		texture_size.width,
		texture_size.height,
		texture_size.width * 4,
		abgr_format);

	provider->convert(
		buffer->get_data(), abgr_format,
		buffer->get_pitch(),
		CL_Rect(0, 0, screen_size.width, screen_size.height),
		CL_Rect(0, 0, screen_size.width, screen_size.height));

	provider->unlock();
	if (delete_provider) delete provider;

	buffer->lock();

	// Upload to OpenGL:
	glGenTextures(1, &handle);
	glBindTexture(GL_TEXTURE_2D, handle);

	glTexImage2D(
		GL_TEXTURE_2D,          // target
		0,                      // level
		GL_RGBA,                // internalformat
		texture_size.width,		// width
		texture_size.height,	// height
		0,                      // border
		GL_RGBA,                // format
		GL_UNSIGNED_BYTE,       // type
		buffer->get_data());    // texels

	// Texture mode parameters must be setup before creating textures!
	// Dunno if this is a driver bug in both nvidia and my laptops s3.
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE_EXT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE_EXT);
	
	buffer->unlock();
	delete buffer;
}

CL_Texture_OpenGL::~CL_Texture_OpenGL()
{
	glDeleteTextures(1, &handle);
}

//////////////////////////////////////////////////////////////////////////////
// CL_Texture_OpenGL attributes:

//////////////////////////////////////////////////////////////////////////////
// CL_Texture_OpenGL operations:

void CL_Texture_OpenGL::bind(CL_GraphicContext *gc)
{
	glBindTexture(GL_TEXTURE_2D, handle);
}

//////////////////////////////////////////////////////////////////////////////
// CL_Texture_OpenGL implementation:
