/*
	$Id: button_generic.cpp,v 1.45 2003/02/16 11:58:30 mbn Exp $
	
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#include "precomp.h"
#include "button_generic.h"
#include "API/GUI/component_options.h"
#include "API/Display/keys.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_Button_Generic::CL_Button_Generic(
	CL_Button *self,
	const std::string &text,
	bool toggle_mode)
:
	button(self),
	text(text),
	toggled(false),
	toggle_mode(toggle_mode)
{
	slots.connect(button->sig_set_options(), this, &CL_Button_Generic::on_set_options);
	slots.connect(button->sig_input_down(), this, &CL_Button_Generic::on_input_down);
	slots.connect(button->sig_input_up(), this, &CL_Button_Generic::on_input_up);
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_Button_Generic::on_set_options(const CL_ComponentOptions &options)
{
	if(options.exists("text"))
		text = options.get_value("text");
	if(options.exists("toggled"))
		toggled = options.get_value_as_bool("toggled");
	if(options.exists("togglemode"))
		toggle_mode = options.get_value_as_bool("togglemode");
}

void CL_Button_Generic::on_input_down(const CL_InputEvent &key)
{
	if(button->is_mouse_captured())
		return;

	if(key.id != CL_MOUSE_LEFT && key.id != CL_KEY_SPACE)
		return;

	sig_pressed();
	button->capture_mouse();
}

void CL_Button_Generic::on_input_up(const CL_InputEvent &key)
{
	if(button->is_mouse_captured() == false)
		return;

	if(key.id != CL_MOUSE_LEFT && key.id != CL_KEY_SPACE)
		return;

	button->release_mouse();

	bool mouse_over = button->has_mouse_over();
	sig_released();
	if (mouse_over)
	{
		if (toggle_mode)
		{
			toggled = !toggled;
			sig_toggled(toggled);
		}
				
		sig_clicked();
	}
}
