/*
	$Id: component_type.cpp,v 1.4 2003/02/18 21:01:31 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	See http://www.clanlib.org
	------------------------------------------------------------------------
*/

#include "precomp.h"
#include "component_type_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_ComponentType construction:

CL_ComponentType::CL_ComponentType() : impl(new CL_ComponentType_Generic)
{
}

CL_ComponentType::~CL_ComponentType()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// CL_ComponentType attributes:

std::map<std::string, CL_ComponentType *> CL_ComponentType::component_types;

const std::map<std::string, CL_ComponentType::SOptionType> &CL_ComponentType::get_options() const
{
	return impl->options;
}

bool CL_ComponentType::has_option(const std::string &option_name) const
{
	return impl->options.find(option_name) != impl->options.end();
}

CL_ComponentType::EDatatype CL_ComponentType::get_data_type(const std::string &option_name) const
{
	std::map<std::string, SOptionType>::iterator it = impl->options.find(option_name);
	if (it == impl->options.end()) return STRING; // throw error?
	return it->second.type;
}

std::string CL_ComponentType::get_default_value(const std::string &option_name) const
{
	std::map<std::string, SOptionType>::iterator it = impl->options.find(option_name);
	if (it == impl->options.end()) return std::string(); // throw error?
	return it->second.default_value;
}

CL_ComponentType::EDatatype CL_ComponentType::get_data_type(const std::string &type, const std::string &option_name)
{
	std::map<std::string, CL_ComponentType *>::const_iterator it(component_types.find(type));
	if (it == component_types.end()) cl_throw_error(std::string("Could not find component type ") + type);

	return it->second->get_data_type(option_name);
}

std::string CL_ComponentType::get_default_value(const std::string &type, const std::string &option_name)
{
	std::map<std::string, CL_ComponentType *>::const_iterator it(component_types.find(type));
	if (it == component_types.end()) cl_throw_error(std::string("Could not find component type ") + type);

	return it->second->get_default_value(option_name);
}

/////////////////////////////////////////////////////////////////////////////
// CL_ComponentType operations:

CL_Component *CL_ComponentType::create_component(
	const std::string &type,
	CL_Component *parent,
	CL_StyleManager *style)
{
	std::map<std::string, CL_ComponentType *>::const_iterator it(component_types.find(type));
	if (it == component_types.end()) cl_throw_error(std::string("Failed to create component of type ") + type);
	
	return (*it).second->create_component(parent, style);
}

void CL_ComponentType::set_option(const std::string &name, EDatatype datatype, const std::string &value)
{
	impl->options[name].type = datatype;
	impl->options[name].default_value = value;
}

/////////////////////////////////////////////////////////////////////////////
//  CL_ComponentType implementation:
