/*
	$Id: inputdialog_generic.cpp,v 1.3 2003/02/25 21:08:37 sphair Exp $
	
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#include "precomp.h"
#include "inputdialog_generic.h"
#include "API/GUI/component_options.h"
#include "API/GUI/stylemanager.h"
#include "API/Display/display.h"
#include "API/Core/Resources/resourcetype_integer.h"
#include "API/Core/Resources/resourcetype_boolean.h"

// private namespace for constants
namespace {
	int const space_between_buttons = 10;
	int const space_between_title_and_inputbox = 10;
	int const space_between_inputboxes = 10;
	int const space_between_inputbox_and_buttons = 20;
	int const space_between_buttons_and_bottom = 10;
	int const space_between_text_and_left_edge = 10;
	int const space_between_text_and_right_edge = 10;
	int const space_between_label_and_input_box = 10;
} // end of private namespace for constants

CL_InputDialog_Generic::CL_InputDialog_Generic(
	CL_InputDialog *self,
	const std::string &title,
	const std::string &button1,
	const std::string &button2,
	const std::string &button3)
:
	inputdialog(self),
	result_button(-1)
{
	CL_ResourceManager * resources = inputdialog->get_style_manager()->get_resources();

	bool const is_fixed_size_buttons = CL_Boolean("InputDialog/fixed_size_buttons",
		resources, false);
	int min_button_width = CL_Integer("InputDialog/min_button_width",
		resources, 80);
	int min_button_height = CL_Integer("InputDialog/min_button_height",
		resources, 20);

	int width = CL_Integer("InputDialog/min_width", resources, 150);

	// Calculating height of text
	int height = 0;

	inputdialog->set_title(title);

	// We need to know how many space get client_area in 
	// CL_Window, and then we can calculate offsets from edges
	CL_Rect const msgbox_rc(
		CL_Display::get_width()  / 2 - 400 / 2,
		CL_Display::get_height() / 2 - 400 / 2,
		CL_Display::get_width()  / 2 + 400 / 2,
		CL_Display::get_height() / 2 + 400 / 2);

	inputdialog->set_position(msgbox_rc);

	CL_Rect const client_area_rc =
		inputdialog->get_client_area()->get_position();

	// Now calculate edges offsets
	int const left_edge_offset   = client_area_rc.left;
	int const top_edge_offset    = client_area_rc.top;
	int const right_edge_offset  = msgbox_rc.right - client_area_rc.right - msgbox_rc.left;
	int const bottom_edge_offset = msgbox_rc.bottom - client_area_rc.bottom - msgbox_rc.top; 	int num_buttons = 0;
	if (!button1.empty())
		num_buttons = 1;
	if (!button2.empty())
		num_buttons = 2;
	if (!button3.empty())
		num_buttons = 3;

	height += space_between_inputbox_and_buttons;
	int const btn_pos_y = height;
	height += space_between_buttons_and_bottom;

	switch(num_buttons) {
	case 1:
		{
			if (is_fixed_size_buttons)
			{
				button[0] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button1,
					inputdialog->get_client_area());
				if (button[0]->get_width() > min_button_width)
					min_button_width = button[0]->get_width();
				if (button[0]->get_height() > min_button_height)
					min_button_height = button[0]->get_height();

				button[0]->set_width(min_button_width);
				button[0]->set_height(min_button_height);
			}
			else
			{
				button[0] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button1,
					inputdialog->get_client_area());
			}

			if (width < button[0]->get_width())
				width = button[0]->get_width();

			width += space_between_text_and_left_edge + space_between_text_and_right_edge;
			
			button[0]->set_position(
				width / 2 - button[0]->get_width() / 2,
				btn_pos_y);

			slot_button[0] = button[0]->sig_clicked().connect(
				this, &CL_InputDialog_Generic::on_button, 0);

			// add buttons height to height
			height += button[0]->get_height();
		}
		break;
	case 2:
		{
			if (is_fixed_size_buttons)
			{
				button[0] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button1,
					inputdialog->get_client_area());
				button[1] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button2,
					inputdialog->get_client_area());
				
				// calculate min width and height of buttons,
				// if some button is bigger than min size
				for(int i = 0; i < 2; ++i)
				{
					if (button[i]->get_width() > min_button_width)
						min_button_width = button[i]->get_width();
					if (button[i]->get_height() > min_button_height)
						min_button_height = button[i]->get_height();
				}
				// set buttons size to max width and height.
				// and use 'j' for msvc workaround :(
				for(int j = 0; j < 2; ++j)
				{
					button[j]->set_width(min_button_width);
					button[j]->set_height(min_button_height);
				}
			}
			else
			{
				button[0] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button1,
					inputdialog->get_client_area());
				button[1] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button2,
					inputdialog->get_client_area());
			}

			int const total_buttons_size =
				button[0]->get_width() +
				space_between_buttons  +
				button[1]->get_width();

			if (width < total_buttons_size)
				width = total_buttons_size;

			width += space_between_text_and_left_edge + space_between_text_and_right_edge;

			button[0]->set_position(
				width / 2 - total_buttons_size / 2,
				btn_pos_y);
			button[1]->set_position(
				width / 2 - total_buttons_size / 2 +
				button[0]->get_width() + space_between_buttons,
				btn_pos_y);

			slot_button[0] = button[0]->sig_clicked().connect(
				this, &CL_InputDialog_Generic::on_button, 0);
			slot_button[1] = button[1]->sig_clicked().connect(
				this, &CL_InputDialog_Generic::on_button, 1);

			// add buttons height to height
			height += button[0]->get_height();
		}
		break;
	case 3:
		{
			if (is_fixed_size_buttons)
			{
				button[0] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button1,
					inputdialog->get_client_area());
				button[1] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button2,
					inputdialog->get_client_area());
				button[2] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button3,
					inputdialog->get_client_area());
				
				// calculate min width and height of buttons,
				// if some button is bigger than min size
				for(int i = 0; i < 3; ++i)
				{
					if (button[i]->get_width() > min_button_width)
						min_button_width = button[i]->get_width();
					if (button[i]->get_height() > min_button_height)
						min_button_height = button[i]->get_height();
				}
				// set buttons size to max width and height
				for(int j = 0; j < 3; ++j)
				{
					button[j]->set_width(min_button_width);
					button[j]->set_height(min_button_height);
				}
			}
			else
			{
				button[0] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button1,
					inputdialog->get_client_area());
				button[1] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button2,
					inputdialog->get_client_area());
				button[2] = new CL_Button(
					CL_Point(width / 2, btn_pos_y),
					button3,
					inputdialog->get_client_area());
			}

			int const total_buttons_size =
				button[0]->get_width() + space_between_buttons +
				button[1]->get_width() + space_between_buttons +
				button[2]->get_width();

			if (width < total_buttons_size)
				width = total_buttons_size;
			
			width += space_between_text_and_left_edge + space_between_text_and_right_edge;

			button[0]->set_position(
				width / 2 - total_buttons_size / 2, btn_pos_y);
			button[1]->set_position(
				width / 2 - total_buttons_size / 2 +
				button[0]->get_width() + space_between_buttons,
				btn_pos_y);
			button[2]->set_position(
				width / 2 - total_buttons_size / 2 +
					button[0]->get_width() + space_between_buttons +
					button[1]->get_width() + space_between_buttons,
				btn_pos_y);

			slot_button[0] = button[0]->sig_clicked().connect(
				this, &CL_InputDialog_Generic::on_button, 0);
			slot_button[1] = button[1]->sig_clicked().connect(
				this, &CL_InputDialog_Generic::on_button, 1);
			slot_button[2] = button[2]->sig_clicked().connect(
				this, &CL_InputDialog_Generic::on_button, 2);

			// add buttons height to height
			height += button[0]->get_height();
		}
		break;
	}
	button_count = num_buttons;

	// Calc and set real dialog position
	width += left_edge_offset + right_edge_offset;

	height += top_edge_offset + bottom_edge_offset;

	inputdialog->set_position(CL_Rect(
		CL_Display::get_width()  / 2 - width / 2,
		CL_Display::get_height() / 2 - height / 2,
		CL_Display::get_width()  / 2 + width / 2,
		CL_Display::get_height() / 2 + height / 2));
}

CL_InputDialog_Generic::~CL_InputDialog_Generic()
{
	for(int i = 0; i < button_count; ++i)
		delete button[i];
	for(size_t j = 0; j < labels.size(); ++j)
		delete labels[j];
	for(size_t k = 0; k < input_boxes.size(); ++k)
		delete input_boxes[k];
}

void CL_InputDialog_Generic::on_button(int button)
{
	result_button = button;
	inputdialog->quit();
}

CL_InputBox * CL_InputDialog_Generic::add_input_box(const std::string &label,
	const std::string &default_value,
	int inputbox_width)
{
	int width = inputdialog->get_client_area()->get_width() -
		space_between_text_and_left_edge - space_between_text_and_right_edge;
	int height = inputdialog->get_client_area()->get_height();

	// We need to know how many space get client_area in 
	// CL_Window, and then we can calculate offsets from edges
	CL_Rect const msgbox_rc(
		CL_Display::get_width()  / 2 - 400 / 2,
		CL_Display::get_height() / 2 - 400 / 2,
		CL_Display::get_width()  / 2 + 400 / 2,
		CL_Display::get_height() / 2 + 400 / 2);

	inputdialog->set_position(msgbox_rc);

	CL_Rect const client_area_rc =
		inputdialog->get_client_area()->get_position();

	// Now calculate edges offsets
	int const left_edge_offset   = client_area_rc.left;
	int const top_edge_offset    = client_area_rc.top;
	int const right_edge_offset  = msgbox_rc.right - client_area_rc.right - msgbox_rc.left;
	int const bottom_edge_offset = msgbox_rc.bottom - client_area_rc.bottom - msgbox_rc.top;

	// Calculating height of all input boxes, which are allready
	// in dialog

	if (button_count > 0)
		// If there some buttons allready, use the button pos to calculate height
		// of all inputboxes
		height = button[0]->get_position().top - space_between_inputbox_and_buttons;
	else
		// Otherwise calculate from client_area pos and constants
		height -= space_between_inputbox_and_buttons + space_between_buttons_and_bottom;

	// If there allready some input boxes, add space between last inputbox
	if (!input_boxes.empty())
		height += space_between_inputboxes;
	else
		height += space_between_title_and_inputbox;

	if (!label.empty())
		labels.push_back(new CL_Label(
			CL_Point(space_between_text_and_left_edge, height),
			label,
			inputdialog->get_client_area()));

	int new_input_box_offset = space_between_text_and_left_edge;
	if (!label.empty())
		new_input_box_offset += labels.back()->get_width() +
			space_between_label_and_input_box;

	// msvc workaround for loop
	{
		// Check all other input boxes positions.
		for(size_t pos = 0; pos < input_boxes.size(); ++pos)
		{
			if (input_boxes[pos]->get_position().left > new_input_box_offset)
				new_input_box_offset = input_boxes[pos]->get_position().left;
		}
	}
	
	// add new input box to InputDialog
	input_boxes.push_back(new CL_InputBox(
		default_value,
		inputdialog->get_client_area()));

	input_boxes.back()->set_position(new_input_box_offset, height);
	input_boxes.back()->set_width(inputbox_width);
	
	if (new_input_box_offset + inputbox_width > width)
		width = new_input_box_offset + inputbox_width;

	// msvc workaround for loop
	{
		// Reposition all input boxes
		for(size_t pos = 0; pos < input_boxes.size(); ++pos)
		{
			input_boxes[pos]->set_position(new_input_box_offset,
				input_boxes[pos]->get_position().top);
			if (width < input_boxes[pos]->get_position().right)
				width = input_boxes[pos]->get_position().right;
		}
	}

	height += input_boxes.back()->get_height() + space_between_inputbox_and_buttons;
	int const btn_pos_y = height;
	height += space_between_buttons_and_bottom;

	width += space_between_text_and_left_edge + space_between_text_and_right_edge;

	switch(button_count) {
	case 1:
		{
			button[0]->set_position(
				width / 2 - button[0]->get_width() / 2,
				btn_pos_y);

			// add buttons height to height
			height += button[0]->get_height();
		}
		break;
	case 2:
		{
			int const total_buttons_size =
				button[0]->get_width() +
				space_between_buttons  +
				button[1]->get_width();

			button[0]->set_position(
				width / 2 - total_buttons_size / 2,
				btn_pos_y);
			button[1]->set_position(
				width / 2 - total_buttons_size / 2 +
				button[0]->get_width() + space_between_buttons,
				btn_pos_y);

			// add buttons height to height
			height += button[0]->get_height();
		}
		break;
	case 3:
		{
			int const total_buttons_size =
				button[0]->get_width() + space_between_buttons +
				button[1]->get_width() + space_between_buttons +
				button[2]->get_width();

			button[0]->set_position(
				width / 2 - total_buttons_size / 2, btn_pos_y);
			button[1]->set_position(
				width / 2 - total_buttons_size / 2 +
				button[0]->get_width() + space_between_buttons,
				btn_pos_y);
			button[2]->set_position(
				width / 2 - total_buttons_size / 2 +
					button[0]->get_width() + space_between_buttons +
					button[1]->get_width() + space_between_buttons,
				btn_pos_y);

			// add buttons height to height
			height += button[0]->get_height();
		}
		break;
	}

	// Calc and set real dialog position
	width += left_edge_offset + right_edge_offset;

	height += top_edge_offset + bottom_edge_offset;

	inputdialog->set_position(CL_Rect(
		CL_Display::get_width()  / 2 - width / 2,
		CL_Display::get_height() / 2 - height / 2,
		CL_Display::get_width()  / 2 + width / 2,
		CL_Display::get_height() / 2 + height / 2));

	return input_boxes.back();
}
