/*
	$Id: menudata.cpp,v 1.13 2002/04/06 11:10:55 grumbel Exp $
	
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#include "precomp.h"

#include "API/GUI/label.h"
#include "API/GUI/menudata.h"
#include "API/GUI/menunode.h"
#include "API/GUI/popupmenu.h"
#include "menudata_generic.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_MenuData::CL_MenuData(CL_PopupMenu *popupmenu)
{
	impl = new CL_MenuData_Generic(this, popupmenu);
}

CL_MenuData::~CL_MenuData()
{
	clear();
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

const std::list<CL_MenuNode *> &CL_MenuData::get_items() const
{
	return impl->items;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

CL_MenuNode *CL_MenuData::insert_item(const std::string &text, int index)
{
	return insert_item(text, NULL, index);
}

CL_MenuNode *CL_MenuData::insert_item(const std::string &text, CL_PopupMenu *submenu, int index)
{
	CL_Component *new_component = new CL_Label(text, impl->popupmenu->get_client_area());

	return insert_item(new_component, submenu, true, index);
}

CL_MenuNode *CL_MenuData::insert_item(CL_Component *new_component, bool delete_component, int index)
{
	return insert_item(new_component, NULL, delete_component, index);
}

CL_MenuNode *CL_MenuData::insert_item(CL_Component *new_component, CL_PopupMenu *submenu, bool delete_component, int index)
{
	// Create menu node
	CL_MenuNode *node = new CL_MenuNode(submenu);
	impl->items.push_back(node);

	// Add component to hierarchy
	impl->popupmenu->get_client_area()->add_child(new_component);
	node->set_component(new_component, delete_component);

	// Make sure components get positioned correctly
	impl->popupmenu->sig_resize()(0,0);

	return node;
}

void CL_MenuData::clear()
{
	std::list<CL_MenuNode *>::iterator it;
	for (it = impl->items.begin(); it != impl->items.end(); ++it)
		delete (*it);

	impl->items.clear();
}
