/*
	$Id: checkbox_boring.cpp,v 1.1 2003/02/18 14:25:10 sphair Exp $
	
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#include "API/Display/surface.h"
#include "API/Display/font.h"
#include "API/Core/Resources/resource_manager.h"
#include "API/GUIStyleBoring/checkbox_boring.h"
#include "API/GUIStyleBoring/stylemanager_boring.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_CheckBox_Boring::CL_CheckBox_Boring(
	CL_CheckBox *checkbox,
	CL_StyleManager_Boring *style)
: 
	CL_ComponentStyle(checkbox),
	checkbox(checkbox),
	style(style),
	sur_checked(0), delete_sur_checked(false),
	sur_unchecked(0), delete_sur_unchecked(false),
	sur_checked_disabled(0), delete_sur_checked_disabled(false),
	sur_unchecked_disabled(0), delete_sur_unchecked_disabled(false),
	font(0), delete_font(false)
{
	resources = style->get_resources();

	if(resources->exists("CheckBox/surface_checked"))
		sur_checked = new CL_Surface("CheckBox/surface_checked", resources);
	if(resources->exists("CheckBox/surface_unchecked"))
		sur_unchecked = new CL_Surface("CheckBox/surface_unchecked", resources);
	if(resources->exists("CheckBox/surface_checked_disabled"))
		sur_checked_disabled = new CL_Surface("CheckBox/surface_checked_disabled", resources);
	if(resources->exists("CheckBox/surface_unchecked_disabled"))
		sur_unchecked_disabled = new CL_Surface("CheckBox/surface_unchecked_disabled", resources);
	if(resources->exists("CheckBox/font"))
		font = new CL_Font("CheckBox/font", resources);

	slot_set_options = checkbox->sig_set_options().connect(
		this, &CL_CheckBox_Boring::on_set_options);
	slot_paint = checkbox->sig_paint().connect_virtual(
		this, &CL_CheckBox_Boring::on_paint);
	slot_get_preferred_size = checkbox->sig_get_preferred_size().connect(
		this, &CL_CheckBox_Boring::on_get_preferred_size);
}

CL_CheckBox_Boring::~CL_CheckBox_Boring()
{
	if(delete_font && font)
		delete font;
	if(delete_sur_checked && sur_checked)
		delete sur_checked;
	if(delete_sur_unchecked && sur_unchecked)
		delete sur_unchecked;
	if(delete_sur_checked_disabled && sur_checked_disabled)
		delete sur_checked_disabled;
	if(delete_sur_unchecked_disabled && sur_unchecked_disabled)
		delete sur_unchecked_disabled;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_CheckBox_Boring::set_font(CL_Font *_font, bool _delete_font)
{
	if(delete_font && font)
		delete font;
	font = _font;
	delete_font = _delete_font;
}

void CL_CheckBox_Boring::set_surface_checked(CL_Surface *_surface, bool _delete_surface)
{
	if(delete_sur_checked && sur_checked)
		delete sur_checked;
	sur_checked = _surface;
	delete_sur_checked = _delete_surface;
}

void CL_CheckBox_Boring::set_surface_unchecked(CL_Surface *_surface, bool _delete_surface)
{
	if(delete_sur_unchecked && sur_unchecked)
		delete sur_unchecked;
	sur_unchecked = _surface;
	delete_sur_unchecked = _delete_surface;
}

void CL_CheckBox_Boring::set_surface_checked_disabled(CL_Surface *_surface, bool _delete_surface)
{
	if(delete_sur_checked_disabled && sur_checked_disabled)
		delete sur_checked_disabled;
	sur_checked_disabled = _surface;
	delete_sur_checked_disabled = _delete_surface;
}

void CL_CheckBox_Boring::set_surface_unchecked_disabled(CL_Surface *_surface, bool _delete_surface)
{
	if(delete_sur_unchecked_disabled && sur_unchecked_disabled)
		delete sur_unchecked_disabled;
	sur_unchecked_disabled = _surface;
	delete_sur_unchecked_disabled = _delete_surface;
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_CheckBox_Boring::on_set_options(const CL_ComponentOptions &options)
{
	if(options.exists("surface_checked"))
		set_surface_checked(new CL_Surface(options.get_value("surface_checked").c_str(), resources), true);
	if(options.exists("surface_unchecked"))
		set_surface_unchecked(new CL_Surface(options.get_value("surface_unchecked").c_str(), resources), true);
	if(options.exists("surface_checked_disabled"))
		set_surface_checked_disabled(new CL_Surface(options.get_value("surface_checked_disabled").c_str(), resources), true);
	if(options.exists("surface_unchecked_disabled"))
		set_surface_unchecked_disabled(new CL_Surface(options.get_value("surface_checked_disabled").c_str(), resources), true);
	if(options.exists("font"))
		set_font(new CL_Font(options.get_value("font").c_str(), resources), true);
}

void CL_CheckBox_Boring::on_get_preferred_size(CL_Point &size)
{
	CL_Surface *s = sur_checked;
	size.x = s->get_width() + font->get_width(checkbox->get_text()) + 8;
	size.y = font->get_height(checkbox->get_text()) + 4;
	if(size.y < (int)s->get_height())
		size.y = s->get_height();
}

void CL_CheckBox_Boring::on_paint(CL_SlotParent_v0 &super)
{
	CL_Surface *show_surface;

	if (checkbox->is_enabled())
	{
		if (checkbox->is_checked())
			show_surface = sur_checked;
		else
			show_surface = sur_unchecked;

		// Focus
		if(checkbox->has_focus())
			style->fill_rect(0, 0, checkbox->get_width(), show_surface->get_height(), GUICOLOR_FOCUS);
	}
	else
	{
		if (checkbox->is_checked())
			show_surface = sur_checked_disabled;
		else
			show_surface = sur_unchecked_disabled;
	}

	show_surface->draw(0, 0);
	if (checkbox->get_text().length() > 0)
		font->draw(4 + show_surface->get_width(), 0, checkbox->get_text());
}
