/*
	$Id: button_silver.cpp,v 1.8 2003/02/23 20:18:55 sphair Exp $
	
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#include "API/Display/display.h"
#include "API/Display/surface.h"
#include "API/Display/font.h"
#include "API/Core/Resources/resource_manager.h"
#include "API/Core/Math/rect.h"
#include "API/GUIStyleSilver/stylemanager_silver.h"
#include "API/GUIStyleSilver/button_silver.h"
#include "API/GUI/gui_manager.h"
#include "API/GUI/component_options.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_Button_Silver::CL_Button_Silver(
	CL_Button *button,
	CL_StyleManager_Silver *style)
:
	CL_ComponentStyle(button),
	button(button),
	style(style),
	sur_up(0), delete_sur_up(false),
	sur_down(0), delete_sur_down(false),
	sur_highlighted(0), delete_sur_highlighted(false),
	sur_disabled(0), delete_sur_disabled(false),
	font(0), delete_font(false),
	font_disabled(0), delete_font_disabled(false),
	down(false)
{
	resources = style->get_resources();

	if(resources->exists("Button/surface_up"))
		set_surface_up(new CL_Surface("Button/surface_up", resources), true);
	if(resources->exists("Button/surface_down"))
		set_surface_down(new CL_Surface("Button/surface_down", resources), true);
	if(resources->exists("Button/surface_highlighted"))
		set_surface_highlighted(new CL_Surface("Button/surface_highlighted", resources), true);
	if(resources->exists("Button/surface_disabled"))
		set_surface_disabled(new CL_Surface("Button/surface_disabled", resources), true);
	if(resources->exists("Button/font"))
		set_font(new CL_Font("Button/font", resources), true);
	if(resources->exists("Button/font_disabled"))
		set_font_disabled(new CL_Font("Button/font_disabled", resources), true);

	slot_set_options = button->sig_set_options().connect(
		this, &CL_Button_Silver::on_set_options);
	slot_paint = button->sig_paint().connect(
		this, &CL_Button_Silver::on_paint);
	slot_get_preferred_size = button->sig_get_preferred_size().connect(
		this, &CL_Button_Silver::on_get_preferred_size);
	slot_pressed = button->sig_pressed().connect(
		this, &CL_Button_Silver::on_pressed);
	slot_released = button->sig_released().connect(
		this, &CL_Button_Silver::on_released);
}

CL_Button_Silver::~CL_Button_Silver()
{
	if(delete_font && font)
		delete font;
	if(delete_font_disabled && font_disabled)
		delete font_disabled;
	if(delete_sur_up && sur_up)
		delete sur_up;
	if(delete_sur_down && sur_down)
		delete sur_down;
	if(delete_sur_highlighted && sur_highlighted)
		delete sur_highlighted;
	if(delete_sur_disabled && sur_disabled)
		delete sur_disabled;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_Button_Silver::set_font(CL_Font *_font, bool _delete_font)
{
	if(delete_font && font)
		delete font;
	font = _font;
	delete_font = _delete_font;
}

void CL_Button_Silver::set_font_disabled(CL_Font *_font, bool _delete_font)
{
	if(delete_font_disabled && font_disabled)
		delete font_disabled;
	font_disabled = _font;
	delete_font_disabled = _delete_font;
}

void CL_Button_Silver::set_surface_up(CL_Surface *surface, bool delete_surface)
{
	if(delete_sur_up && sur_up)
		delete sur_up;
	sur_up = surface;
	delete_sur_up = delete_surface;
}

void CL_Button_Silver::set_surface_down(CL_Surface *surface, bool delete_surface)
{
	if(delete_sur_down && sur_down)
		delete sur_down;
	sur_down = surface;
	delete_sur_down = delete_surface;
}

void CL_Button_Silver::set_surface_highlighted(CL_Surface *surface, bool delete_surface)
{
	if(delete_sur_highlighted && sur_highlighted)
		delete sur_highlighted;
	sur_highlighted = surface;
	delete_sur_highlighted = delete_surface;
}

void CL_Button_Silver::set_surface_disabled(CL_Surface *surface, bool delete_surface)
{
	if(delete_sur_disabled && sur_disabled)
		delete sur_disabled;
	sur_disabled = surface;
	delete_sur_disabled = delete_surface;
}

void CL_Button_Silver::set_scrollbar_increase_mode()
{
}

void CL_Button_Silver::set_scrollbar_decrease_mode()
{
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_Button_Silver::on_set_options(const CL_ComponentOptions &options)
{
	if(options.exists("surface_up"))
		set_surface_up(new CL_Surface(options.get_value("surface_up").c_str(), resources), true);
	if(options.exists("surface_down"))
		set_surface_down(new CL_Surface(options.get_value("surface_down").c_str(), resources), true);
	if(options.exists("surface_disabled"))
		set_surface_disabled(new CL_Surface(options.get_value("surface_disabled").c_str(), resources), true);
	if(options.exists("surface_highlighted"))
		set_surface_highlighted(new CL_Surface(options.get_value("surface_highlighted").c_str(), resources), true);
	if(options.exists("font"))
		set_font(new CL_Font(options.get_value("font").c_str(), resources), true);
	if(options.exists("font_disabled"))
		set_font_disabled(new CL_Font(options.get_value("font_disabled").c_str(), resources), true);
}

void CL_Button_Silver::on_get_preferred_size(CL_Point &size)
{
	if(sur_up)
	{
		size.x = sur_up->get_width();
		size.y = sur_up->get_height();
	}
	else
	{
		if(font)
		{
			size.x = font->get_width(button->get_text()) + 8;
			size.y = font->get_height(button->get_text()) + 4;
		}
	}
}

void CL_Button_Silver::on_paint()
{
	int text_width = 0;
	int text_height = 0;

	if(font)
	{
		text_width = font->get_width(button->get_text().c_str());
		text_height = font->get_height();
	}

	int button_width = button->get_width();
	int button_height = button->get_height();

	int x_pos = (button_width - text_width) / 2;
	int y_pos = (button_height - text_height) / 2;

	int width = button->get_width();
	int height = button->get_height();

	if(button->is_enabled() == false)
	{
		if(sur_disabled)
			sur_disabled->draw(0, 0);
		else
		{
			// Fill
//			style->fill_rect(2, 2, width - 2, height - 2, GUICOLOR_BUTTON_DISABLED);

			// Outline
//			style->draw_rect(0, 0, width, height, GUICOLOR_DARK_OUTLINE_DISABLED);

			// Shade box
//			style->draw_box(1, 1, width - 1 , height - 1, GUICOLOR_BRIGHT_SHADE_DISABLED, GUICOLOR_DARK_SHADE_DISABLED);
		}

		if(font_disabled)
			font_disabled->draw(x_pos, y_pos, button->get_text());
		else if(font)
			font->draw(x_pos, y_pos, button->get_text());
	}
	else 
	{
		bool being_clicked = button->has_mouse_over() ? down : false;
		if((button->is_down() && !being_clicked) || (!button->is_down() && being_clicked))
		{
			if(sur_down)
				sur_down->draw(0, 0);
			else
			{
				CL_Display::fill_rect(
					CL_Rect(2, 2, width, height), 
					CL_Gradient(CL_Color::white, CL_Color::white, CL_Color(230, 235, 240), CL_Color(230, 235, 240)));

				CL_Display::fill_rect(
					CL_Rect(4, 4, width - 2, height - 2),
					CL_Gradient(CL_Color(212, 217, 222), CL_Color(212, 217, 222), CL_Color(250,250,250), CL_Color(250,250,250)));

				CL_Display::draw_rect(CL_Rect(1, 1, width, height), CL_Color(73, 94, 120));
				CL_Display::draw_rect(CL_Rect(0, 0, width, height), CL_Color(128, 142, 159));
			}

			if(font)
				font->draw(x_pos + 1, y_pos + 1, button->get_text());
		}
		else
		{
			bool need_highlight = false;
			if (button->has_mouse_over())
				if (!button->get_gui_manager() ||
					(!button->get_gui_manager()->get_modal_component() ||
					button->has_modal_parent()))
					need_highlight = true;

			if(sur_highlighted && need_highlight)
				sur_highlighted->draw(0, 0);
			else if(sur_up)
				sur_up->draw(0, 0);
			else
			{
				CL_Display::draw_rect(CL_Rect(0, 0, width, height), CL_Color(128, 142, 159));

				CL_Display::fill_rect(
					CL_Rect(1, 1, width - 1, height - 1), 
					CL_Gradient(CL_Color::white, CL_Color::white, CL_Color(230, 235, 240), CL_Color(230, 235, 240)));

				CL_Display::fill_rect(
					CL_Rect(3, 3, width - 3, height - 3),
					CL_Gradient(CL_Color(217, 222, 227), CL_Color(217, 222, 227), CL_Color::white, CL_Color::white));

				// Focus
				if(button->has_focus())
				{
					CL_Display::draw_rect(
						CL_Rect(2, 2, width - 2, height - 2), CL_Color(254, 207, 147));
					CL_Display::draw_rect(
						CL_Rect(3, 3, width - 3, height - 3), CL_Color(255, 147, 22));
				}
			}

			if(font)
				font->draw(x_pos, y_pos, button->get_text());
		}
	}
}

void CL_Button_Silver::on_pressed()
{
	down = true;
}

void CL_Button_Silver::on_released()
{
	down = false;
}
