/*
	$Id: image_silver.cpp,v 1.1 2003/02/18 14:25:10 sphair Exp $
	
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

#include "API/Display/surface.h"
#include "API/Core/Resources/resource_manager.h"
#include "API/GUIStyleSilver/image_silver.h"
#include "API/GUIStyleSilver/stylemanager_silver.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_Image_Silver::CL_Image_Silver(
	CL_Image *image,
	CL_StyleManager_Silver *style) 
:
	CL_ComponentStyle(image),
	image(image)
{
	resources = style->get_resources();

	slot_paint = image->sig_paint().connect(this, &CL_Image_Silver::on_paint);
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_Image_Silver::on_paint()
{
	CL_Surface *surface = image->get_surface();

	if(surface == 0)
		return;

	int width = image->get_width();
	int height = image->get_height();

	CL_Image::Mode mode = image->get_mode();

	switch (mode)
	{
	case CL_Image::normal:
		surface->draw(0, 0);
		break;

	case CL_Image::center:
		surface->draw(
			(width - surface->get_width()) / 2,
			(height - surface->get_height()) / 2);
		break;
		
	case CL_Image::stretch:
		surface->draw(CL_Rect(0, 0, width, height));
		break;

	case CL_Image::tile:
		{
			for (int y = 0; y < height; y += surface->get_height())
				for (int x = 0; x < width; x += surface->get_width())
					surface->draw(x, y);
		}
		break;
	}
}
