/*
	$Id: resourcetype_sample.cpp,v 1.11 2002/08/06 12:06:41 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Sample resource type

*/

#ifdef WIN32
#pragma warning (disable:4786)
#endif

#include "resourcetype_sample.h"
#include "API/Core/System/cl_assert.h"
#include "API/Core/System/error.h"
#include "API/Core/Resources/resource.h"
#include "API/Core/Resources/resourceoptions.h"
#include <API/Core/IOData/inputsource.h>
#include <API/Core/IOData/inputsource_provider.h>
#include <API/Core/IOData/outputsource.h>
#include "API/Core/IOData/outputsource_provider.h"
#include <API/Sound/soundbuffer.h>
#include <API/Sound/static_soundprovider.h>
#include <API/Sound/stream_soundprovider.h>
#include <API/Sound/SoundProviders/static_provider_wave.h>
#include <API/Sound/SoundProviders/stream_provider_wave.h>
#include <API/Sound/SoundProviders/soundprovider_factory.h>

/////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Sample Construction:

CL_ResourceData_Sample::CL_ResourceData_Sample(class CL_Resource &resource)
: CL_ResourceData(resource)
{
	slot_load_file = resource.sig_load_file().connect(this, &CL_ResourceData_Sample::on_load_file);
	slot_load_datafile = resource.sig_load_datafile().connect(this, &CL_ResourceData_Sample::on_load_datafile);
	slot_save_datafile = resource.sig_save_datafile().connect(this, &CL_ResourceData_Sample::on_save_datafile);
	slot_unload = resource.sig_unload().connect(this, &CL_ResourceData_Sample::on_unload);

	resource.attach_data("sample", this);
}

CL_ResourceData_Sample::~CL_ResourceData_Sample()
{
}

/////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Sample Attributes:

/////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Sample Implementation:

void CL_ResourceData_Sample::on_load_file()
{
	CL_Resource resource = get_resource();
	CL_ResourceOptions &options = resource.get_options();

	CL_StreamSoundProvider *stream_provider = 0;
	CL_StaticSoundProvider *static_provider = 0;

	if (options.exists("stream")) // try streamed source first.
	{
		stream_provider = CL_SoundProviderFactory::load_stream(
			resource.get_location(),
			"",
			resource.get_manager().get_resource_provider());
	}

	if (stream_provider == 0)
	{
		static_provider = CL_SoundProviderFactory::load_static(
			resource.get_location(),
			"",
			resource.get_manager().get_resource_provider());

		if (static_provider == 0) // no static source. Try for streamed one.
		{
			stream_provider = CL_SoundProviderFactory::load_stream(
				resource.get_location(),
				"",
				resource.get_manager().get_resource_provider());
		}
	}

	if (static_provider)
	{
		soundbuffer = new CL_SoundBuffer(static_provider, true);
	}
	else if (stream_provider)
	{
		soundbuffer = new CL_SoundBuffer(stream_provider, true);
	}
	else
	{
		throw CL_Error("Unknown sample format");
	}

/*
		soundbuffer = new CL_SoundBuffer(
			new CL_Streamed_WaveSample(
				resource.get_location().c_str(),
				resource.get_manager().get_resource_provider(),
				options.exists("loop")),
			true);
	}
	else
	{
		soundbuffer = new CL_SoundBuffer(
			new CL_Sample(
				resource.get_location().c_str(),
				resource.get_manager().get_resource_provider()),
			true);
	}
*/
}

void CL_ResourceData_Sample::on_load_datafile(class CL_InputSourceProvider *input_provider)
{
	CL_InputSource *input = input_provider->open_source(get_resource().get_name().c_str());

	CL_Resource resource = get_resource();
	CL_ResourceOptions &options = resource.get_options();

	if (options.exists("stream"))
	{
		soundbuffer = new CL_SoundBuffer(
			new CL_Streamed_WaveSample(
				resource.get_name().c_str(),
				resource.get_manager().get_resource_provider(),
				false /*options->exists("loop")*/ ),
			true);
	}
	else
	{
		soundbuffer = new CL_SoundBuffer(
			new CL_Sample(
				resource.get_name().c_str(),
				resource.get_manager().get_resource_provider()),
			true);
	}

	delete input;
}

void CL_ResourceData_Sample::on_save_datafile(class CL_OutputSourceProvider *output_provider)
{
	CL_OutputSource *output = output_provider->open_source(get_resource().get_name().c_str());

	CL_InputSourceProvider *provider = get_resource().get_manager().get_resource_provider();
	cl_assert(provider != NULL);

	CL_InputSource *wavefile = provider->open_source(get_resource().get_location().c_str());
	cl_assert(wavefile != NULL);

	int size_left = wavefile->size();
	while (size_left > 0)
	{
		unsigned char buffer[32768];
		int read = wavefile->read(buffer, 32768);
		size_left -= read;
		output->write(buffer, read);
	}

	delete wavefile;
	delete output;
}

void CL_ResourceData_Sample::on_unload()
{
	delete soundbuffer;
	soundbuffer = NULL;
}
