
#ifdef WIN32
#pragma warning(disable : 4786)
#endif

#include "API/Sound/SoundProviders/soundprovider_factory.h"
#include "API/Sound/SoundProviders/soundprovider_type.h"
#include "API/Core/System/error.h"
#include "API/Core/System/clanstring.h"
#include <iostream>

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProviderFactory attributes:

std::map<std::string, CL_SoundProviderType *> CL_SoundProviderFactory::types;

/////////////////////////////////////////////////////////////////////////////
// CL_SoundProviderFactory operations:

CL_StaticSoundProvider *CL_SoundProviderFactory::load_static(
	const std::string &filename,
	const std::string &type,
	CL_InputSourceProvider *input_provider)
{
	if (type != "")
	{
		if (types.find(type) == types.end()) throw CL_Error("Unknown sound provider type " + type);

		CL_SoundProviderType *factory = types[type];
		return factory->load_static(filename, input_provider);
	}

	// Determine file extension and use it to lookup type.
	std::string ext = CL_String::right(filename, 4);
	if (ext.at(0) == '.') ext = CL_String::right(ext, 3);
	ext = CL_String::to_lower(ext);
	if (types.find(ext) == types.end()) throw CL_Error(std::string("Unknown sound provider type ") + ext.c_str());

	CL_SoundProviderType *factory = types[ext];
	return factory->load_static(filename, input_provider);
}

CL_StreamSoundProvider *CL_SoundProviderFactory::load_stream(
	const std::string &filename,
	const std::string &type,
	CL_InputSourceProvider *input_provider)
{
	if (type != "")
	{
		if (types.find(type) == types.end()) throw CL_Error("Unknown sound provider type " + type);

		CL_SoundProviderType *factory = types[type];
		return factory->load_stream(filename, input_provider);
	}

	// Determine file extension and use it to lookup type.
	std::string ext = CL_String::right(filename, 4);
	if (ext.at(0) == '.') ext = CL_String::right(ext, 3);
	ext = CL_String::to_lower(ext);
	if (types.find(ext) == types.end()) throw CL_Error(std::string("Unknown sound provider type ") + ext.c_str());

	CL_SoundProviderType *factory = types[ext];
	return factory->load_stream(filename, input_provider);
}
