#include <ClanLib/application.h>
#include <ClanLib/core.h>
#include <ClanLib/display.h>
#include <ClanLib/gl.h>
#include <GL/gl.h>
#include <GL/glu.h>

/////////////////////////////////////////////////////////////////////////////
// Application class

class Application : public CL_ClanApplication
{
public:
	int main(int argc, char** argv);

	void sprite_test(CL_DisplayWindow &window);
} app;

/////////////////////////////////////////////////////////////////////////////
// Application main entry point:

int Application::main(int argc, char** argv)
{
	CL_ConsoleWindow debug_console("Debug console");
	debug_console.redirect_stdio();
	try
	{
		CL_SetupCore::init();
		CL_SetupDisplay::init();
		CL_SetupGL::init();

		CL_DisplayWindow window("ClanLib Sprite Test", 800, 600, false, true);

		sprite_test(window);
		
		CL_SetupGL::deinit();
		CL_SetupDisplay::deinit();
		CL_SetupCore::deinit();
	}
	catch (CL_Error error)
	{
		std::cout << "Uncaught error: " << error.message.c_str() << std::endl;
		debug_console.wait_for_key();
	}

	return 0;
}

void Application::sprite_test(CL_DisplayWindow &window)
{
	CL_SpriteDescription desc;
	
	desc.add_frame(new CL_TargaProvider("images/manwalk_0001.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0002.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0003.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0004.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0005.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0006.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0007.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0008.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0009.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0010.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0011.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0012.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0013.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0014.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0015.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0016.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0017.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0018.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0019.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0020.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0021.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0022.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0023.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0024.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0025.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0026.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0027.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0028.tga"), true);
	desc.add_frame(new CL_TargaProvider("images/manwalk_0029.tga"), true);

	CL_Sprite sprite(desc);
	
	CL_GraphicContext *gc = window.get_gc();

	float last_time = (float)CL_System::get_time();

	float colors[26][20][3];
	for(int y=0; y<19; y++)
	{
		for(int x=0; x<26; x++) 
		{
			colors[x][y][0] = (rand() % 100) / 100.0f;
			colors[x][y][1] = (rand() % 100) / 100.0f;
			colors[x][y][2] = (rand() % 100) / 100.0f;
		}
	}

	while (!CL_Keyboard::get_keycode(CL_KEY_ESCAPE))
	{
		// Calculate amount of time since last frame
		float delta_time = (CL_System::get_time() - last_time) / 1000.0f;
		last_time = (float)CL_System::get_time();

		// Rotate sprite based on time
		sprite.rotate(100.0f * delta_time);
		sprite.set_scale(1.0f, 1.0f);
		sprite.set_alignment(origin_top_left);
		
		// Update sprite animation based on time 
		sprite.update(delta_time);

		gc->clear();

		for(int y=0; y<19; y++)
		{
			for(int x=0; x<26; x++) 
			{
				sprite.set_color(colors[x][y][0], colors[x][y][1], colors[x][y][2]);
				sprite.draw(x * 40, y * 40, gc);
			}
		}

		sprite.set_color(1.0f, 1.0f, 1.0f);
		sprite.set_alignment(origin_center);
		sprite.set_scale(2.0f, 2.0f);
		sprite.draw(gc->get_width()/2-100, gc->get_height()/2, gc);
		sprite.draw(gc->get_width()/2+100, gc->get_height()/2, gc);

		window.flip();
		CL_System::keep_alive(10);
	}
}