
#pragma warning(disable:4786)

#include "document.h"
#include "component_node.h"
#include <ClanLib/Core/IOData/inputsource_provider.h>
#include <ClanLib/GUI/filedialog.h>

/////////////////////////////////////////////////////////////////////////////
// Document construction:

Document::Document()
{
}

Document::~Document()
{
}

/////////////////////////////////////////////////////////////////////////////
// Document attributes:


/////////////////////////////////////////////////////////////////////////////
// Document operations:

void Document::clear()
{
	std::list<ComponentNode *>::iterator it(nodes.begin());
	std::list<ComponentNode *>::iterator it_end(nodes.end());
	for (;it!=it_end;++it)
		delete *it;
	nodes.clear();

	sig_document_reloaded.call();
}

void Document::load(const std::string &gui_file)
{
	CL_GUIFileParser parser(
		gui_file, 
		CL_InputSourceProvider::create_file_provider(".")->open_source(gui_file));

	CL_GUIFileParser::component_sort_map_t::iterator it;
	it = parser.get_sorted_components().find(0);
	while (it != parser.get_sorted_components().end())
	{
		// Only enumerate first level components:
		if (it->first != 0) break;

		CL_GUIFileParser::ComponentInfo *info = it->second;
		ComponentNode *node = new ComponentNode(info->name, info->type, info->options, this);
		nodes.push_back(node);

		std::list<CL_GUIFileParser::ComponentInfo *>::iterator it_child;
		for (it_child = info->children.begin(); it_child != info->children.end(); it_child++)
		{
			add_node_child(node, *it_child);
		}

		it++;
	}
	sig_document_reloaded.call();
}

void Document::save(const std::string &gui_file)
{
	std::ofstream fp(gui_file.c_str());

	std::list<ComponentNode *>::iterator it;
	for (it = nodes.begin(); it != nodes.end(); ++it)
	{
		save_node(fp, *it, 0);
	}
}

void Document::save_node(std::ofstream &fp, ComponentNode *node, int indent_level)
{
	std::string indent0(indent_level, '\t');
	std::string indent1(indent_level+1, '\t');

	fp << indent0.c_str() << node->get_type().c_str();
	if (!node->get_name().empty()) fp << " " << node->get_name().c_str();
	fp << std::endl;
	fp << indent0.c_str() << "{" << std::endl;

	CL_ComponentOptions options = node->get_options();
	CL_ComponentOptions::iterator it_opt;
	for (it_opt = options.begin(); it_opt != options.end(); ++it_opt)
	{
		const std::string &name = it_opt->first;
		const std::string &value = it_opt->second;

		fp << indent1.c_str() << name.c_str() << " = \"" << value.c_str() << "\"" << std::endl;
	}

	std::list<ComponentNode *>::iterator it_node;
	for (it_node = node->nodes.begin(); it_node != node->nodes.end(); ++it_node)
	{
		fp << std::endl;
		save_node(fp, *it_node, indent_level+1);
	}
	
	fp << indent0.c_str() << "}" << std::endl;
}

ComponentNode *Document::add_node(
	const std::string &type,
	const std::string &name,
	const CL_ComponentOptions &options,
	ComponentNode *parent)
{
	ComponentNode *node;
	if (parent != NULL)
		node = new ComponentNode(name, type, options, parent);
	else
		node = new ComponentNode(name, type, options, this);

	nodes.push_back(node);
	sig_node_added(node);
	return node;
}

void Document::remove_node(ComponentNode *node)
{
	nodes.remove(node);
	sig_node_removed(node);
	delete node;
}

/////////////////////////////////////////////////////////////////////////////
// Document signals:


/////////////////////////////////////////////////////////////////////////////
// Document implementation:

void Document::add_node_child(ComponentNode *parent, CL_GUIFileParser::ComponentInfo *info)
{
	ComponentNode *node = new ComponentNode(info->name, info->type, info->options, parent);
	parent->nodes.push_back(node);

	std::list<CL_GUIFileParser::ComponentInfo *>::iterator it_child;
	for (it_child = info->children.begin(); it_child != info->children.end(); it_child++)
	{
		add_node_child(node, *it_child);
	}
}
